"""
MIT License

Sugaroid Artificial Inteligence
Chatbot Core
Copyright (c) 2020 Srevin Saju

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""

import nltk
import wikipediaapi
from chatterbot.logic import LogicAdapter
from nltk import word_tokenize
from sugaroid.brain.brain import Neuron
from sugaroid.brain.ooo import Emotion
from sugaroid.sugaroid import SugaroidStatement
try:
    from mediawikiapi import MediaWikiAPI
    media_wiki_found = True
except ModuleNotFoundError:
    media_wiki_found = False


class WikiAdapter(LogicAdapter):
    """
    Handles Wikipedia based questions
    """

    def __init__(self, chatbot, **kwargs):
        # FIXME Add Language support
        super().__init__(chatbot, **kwargs)

    def can_process(self, statement):
        self.text = word_tokenize(str(statement))
        tagged = nltk.pos_tag(self.text)
        q = False
        pr = False
        for i in tagged:
            if i[1] == 'WP' or i[1] == 'WRB':
                q = True
            elif (i[1].startswith('PRP')) and (not i[0] == 'we'):
                pr = True
        if q and (not pr):
            return True
        else:
            return False

    def process(self, statement, additional_response_selection_parameters=None):
        # FIXME: This may printout unrelated data for phrase searches
        response = "I don't know"
        confidence = 0

        emotion = Emotion.neutral
        count = 0
        for i in self.text:
            if i.lower() == 'what':
                count += 1
        if count > 1:
            selected_statement = SugaroidStatement(
                'Nothing ' * count, chatbot=True)
            selected_statement.confidence = 0.99
            emotion = Emotion.seriously
            selected_statement.emotion = emotion
            return selected_statement
        elif ('Srevin' in self.text) or ('srevin' in self.text):
            response = 'Srevin Saju is the creator of Sugaroid bot'
            selected_statement = SugaroidStatement(response, chatbot=True)
            selected_statement.confidence = 1.0
            selected_statement.emotion = emotion
            return selected_statement

        else:
            nlp = self.chatbot.lp
            what = False
            norm = nlp.tokenize(str(statement))
            for i in range(len(norm) - 2):
                if norm[i].tag_ == 'WP':
                    if norm[i].lower_ == "what" or norm[i].lower_ == "who" or norm[i].lower_ == "where":
                        what = True
                    elif norm[i].lower_ == 'how':
                        what = False
                        how = True
                        break
                    else:
                        what = True
                    if what:
                        for j in range(i + 1, len(norm)):
                            if (norm[j].tag_ == 'VBZ') or (norm[j].tag_ == 'DT'):
                                continue
                            else:
                                question = norm[j:]
                                if question.lower_ == 'time':
                                    response = Neuron.gen_time()
                                    confidence = 1.0
                                    stat = True
                                    break
                                else:
                                    if media_wiki_found:
                                        response, confidence, stat = wikipedia_search(
                                            self, question)
                                        break
                                    else:
                                        response = "Seems like MediaWikiAPI is not installed on your PC"
                                        confidence = 0.9
                                        break
                        else:
                            response = 'I am not sure what you are asking for.'
                            confidence = 0.4
                            emotion = Emotion.cry
                            break
                        if response:
                            break
                    else:
                        response = 'I can\'t give a reason for that at the moment. Maybe you might want to search the ' \
                                   'internet.'
                        confidence = 0.4
                        emotion = Emotion.cry_overflow
                        break

                else:
                    response = 'Well. maybe I do not know'
                    confidence = 0

            selected_statement = SugaroidStatement(str(response), chatbot=True)
            selected_statement.confidence = confidence

        selected_statement.emotion = emotion

        return selected_statement


def wikipedia_search(self, question):
    """
    :param self: self object where chatbot object is also located
    :param question:
    :return: A tuple of values:
    (response, confidence, stat)
    stat = Found or not found
    """
    mw = MediaWikiAPI()
    wiki = wikipediaapi.Wikipedia('en')
    a = mw.search(str(question))
    question = str(question)
    if len(a) >= 1:
        cos = self.chatbot.lp.similarity(question.lower(), a[0].lower())
    else:
        return "Oops, the item you wanted to know is not on wikipedia.", 0.9, False
    if cos > 0.9:
        self.chatbot.globals['reversei']['enabled'] = False
        self.chatbot.globals['reversei']['uid'] = False
        response = wiki.page(a[0]).summary
        confidence = cos
        stat = True
    else:
        self.chatbot.globals['reversei']['enabled'] = True
        self.chatbot.globals['reversei']['uid'] = 30000000002
        self.chatbot.globals['reversei']['type'] = int
        self.chatbot.globals['temp_data'] = a
        def bracketize(x): return '\n[{}] {}'.format(x[0]+1, str(x[1]))
        response = "Did you mean any of these {}".format(
            ' '.join([bracketize(x) for x in enumerate(a)]))
        confidence = 1.0
        stat = False

    return response, confidence, stat



"""
MIT License

Sugaroid Artificial Inteligence
Chatbot Core
Copyright (c) 2020 Srevin Saju

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""
import logging
from chatterbot.logic import LogicAdapter
from chatterbot.trainers import ListTrainer
from sugaroid.brain.postprocessor import random_response, any_in
from sugaroid.brain.constants import ASK_AND_YOU_SHALL_RECEIVE, SEEK_AND_YOU_SHALL_FIND, THANK
from sugaroid.sugaroid import SugaroidStatement
from sugaroid.brain.ooo import Emotion


class InterruptAdapter(LogicAdapter):
    __adapter__ = 'interr'

    def __init__(self, chatbot, **kwargs):
        super().__init__(chatbot, **kwargs)
        self.tokenized = None
        self.nn = False

    def can_process(self, statement):

        if self.chatbot.interrupt == 2:
            logging.info("InterruptAdapter: Found Discord")
            self.tokenized = self.chatbot.lp.nlp(str(statement))
            if 6 > len(self.tokenized) > 2:
                for i in self.tokenized:
                    if str(i.tag_).startswith('NNP'):
                        self.nn = i.lemma_
                        if self.nn in self.chatbot.globals['learned']:
                            return False
                return True
            else:
                self.chatbot.interrupt = False
                return False
        elif self.chatbot.interrupt == 1:
            return True
        else:
            return False

    def process(self, statement, additional_response_selection_parameters=None, username=None):
        if self.chatbot.interrupt == 2:
            if self.nn:
                response = "{} {} what is {}" \
                    .format(random_response(ASK_AND_YOU_SHALL_RECEIVE), random_response(SEEK_AND_YOU_SHALL_FIND),
                            self.nn)
                self.chatbot.interrupt = self.nn
            else:
                if username:
                    response = "{} {} what is actually meant in {}'s message?" \
                        .format(
                            random_response(ASK_AND_YOU_SHALL_RECEIVE),
                            random_response(SEEK_AND_YOU_SHALL_FIND),
                            username
                        )

                else:
                    response = "{} {} what is actually meant in the previous message?" \
                        .format(
                            random_response(ASK_AND_YOU_SHALL_RECEIVE),
                            random_response(SEEK_AND_YOU_SHALL_FIND),
                        )
                self.chatbot.interrupt = str(statement)
        else:
            if any_in(['no', 'not', 'later', 'busy', 'nah'], self.tokenized) or \
                    (('next' in self.tokenized or 'another' in self.tokenized) and 'time' in self.tokenized):
                response = 'Ok.'
                self.chatbot.interrupt = False
            else:
                response = random_response(THANK)
                learner = ListTrainer(self.chatbot)
                learner.train([
                    'What is {} ?'.format(self.chatbot.interrupt),
                    str(statement)
                ])
                self.chatbot.globals['learned'].append(self.chatbot.interrupt)
                self.chatbot.interrupt = False
        selected_statement = SugaroidStatement(response, chatbot=True)
        selected_statement.confidence = 9
        emotion = Emotion.lol
        selected_statement.emotion = emotion
        return selected_statement



"""
MIT License

Sugaroid Artificial Intelligence
Chatbot Core
Copyright (c) 2020 Srevin Saju

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""

from difflib import SequenceMatcher
from nltk import pos_tag
from chatterbot.logic import LogicAdapter
from nltk.sentiment import SentimentIntensityAnalyzer
from sugaroid.brain.ooo import Emotion
from sugaroid.brain.preprocessors import normalize
from sugaroid.sugaroid import SugaroidStatement


class BecauseAdapter(LogicAdapter):
    """
    Processes statements which starts with Because or gives a reason
    """

    def __init__(self, chatbot, **kwargs):
        super().__init__(chatbot, **kwargs)
        self.chatbot = chatbot
        self.normalized = None
        self.last_normalized = None
        self.tagged_now = None
        self.tagged_last = None

    def can_process(self, statement):
        self.normalized = normalize(str(statement))

        if 'because' in self.normalized:
            return True
        else:
            return False

    def process(self, statement, additional_response_selection_parameters=None):
        # add emotion output
        adj = None
        verb = None
        confidence = 0.90
        last_response = self.chatbot.globals['history']['total'][-1]
        emotion = Emotion.neutral
        self.last_normalized = normalize(str(last_response))
        if last_response:
            self.tagged_last = pos_tag(self.last_normalized)
            self.tagged_now = pos_tag(self.normalized)
            sm = SequenceMatcher(None, self.tagged_last, self.tagged_now)
            for i in self.tagged_now:
                if i[1] == 'JJ':
                    adj = i[0]
                elif i[1] == 'VB' and (not i[0] == 'be'):
                    verb = i[0]
            if sm.ratio() > 0.5:
                if adj:
                    response = 'Well, Its not a good reason for me to be {}'.format(
                        adj)
                else:
                    response = 'Well, its not a good reason you have told me 😭'
            else:
                if verb:
                    if verb in ['think', 'breath', 'eat', 'hear', 'feel', 'taste']:
                        response = 'Robots are computer devices. I cannot {}'.format(
                            verb.replace('ing', ''))
                        emotion = Emotion.cry
                    else:
                        response = "I may not be able to {}. " \
                                   "This might not be my builtin quality".format(
                                       verb.replace('ing', ''))
                        emotion = Emotion.cry_overflow
                else:
                    if adj:
                        sia = SentimentIntensityAnalyzer()
                        sia_scores = sia.polarity_scores(str(statement))
                        if sia_scores['neu'] == 1:
                            response = 'Ok! Thanks for your feedback'
                            emotion = Emotion.positive
                        elif sia_scores['pos'] > sia_scores['neg']:
                            response = 'I will try to be more {} in future'.format(
                                adj)
                            emotion = Emotion.adorable
                        else:
                            response = 'I will never try to be {}.'.format(adj)
                            emotion = Emotion.angry
                    else:
                        response = 'Are you sure this is the reason? I would love to report to my creator.'
                        self.chatbot.report = True
                        emotion = Emotion.non_expressive_left

        else:
            response = 'Well, I cannot think of saying something. Your conversation began with reason. 🤯'
            emotion = Emotion.angry

        selected_statement = SugaroidStatement(response, chatbot=True)
        selected_statement.confidence = confidence
        selected_statement.emotion = emotion
        selected_statement.adapter = None
        return selected_statement



"""
MIT License

Sugaroid Artificial Intelligence
Chatbot Core
Copyright (c) 2020 Srevin Saju

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""
import logging
from chatterbot.logic import LogicAdapter
from nltk.sentiment import SentimentIntensityAnalyzer
from pyinflect import getInflection
from sugaroid.brain.postprocessor import any_in, random_response
from sugaroid.brain.constants import BYE, DIS_RESPONSES_YOU, CONSOLATION, DIS_RESPONSES_I, DIS_RESPONSES_HIM
from sugaroid.brain.ooo import Emotion
from sugaroid.brain.preprocessors import normalize, spac_token
from sugaroid.sugaroid import SugaroidStatement


class DisAdapter(LogicAdapter):
    """
    A complex algorithm sorting the words beginning with negative based on the probability.
    and achieving a similar confidence ratio of the word percentage.
    The DisAdapter keeps the confidence below 0.5 so that the BestAdapter may find some
    other answer similar to
    """

    def __init__(self, chatbot, **kwargs):
        super().__init__(chatbot, **kwargs)
        self.normalized = None
        self.dis = None

    def can_process(self, statement):
        self.normalized = normalize(str(statement))
        self.dis = None
        for i in self.normalized:
            if i.startswith('dis'):
                self.dis = i
                return True
        else:
            return False

    def process(self, statement, additional_response_selection_parameters=None):
        confidence = 0
        dis_word = False
        if any_in(['distinguish', 'disfigure', 'distinct', 'distinction', 'distant',
                   'distance', 'distribution', 'distilled'], self.normalized):
            confidence = 0
        else:
            logging.info(
                "DisAdapter: Starting Advanced scan. dis_word == {}".format(self.dis)[0])
            dis_word = self.dis[3:]
            logging.info("DisAdapter: Distilled word == {}".format(dis_word))
            sia = SentimentIntensityAnalyzer().polarity_scores(dis_word)
            if dis_word[0] in ['a', 'e', 'i', 'o', 'u', 'g', 'm', 'p']:
                confidence += 0.4
            if 'infect' in dis_word:
                confidence -= 0.3
            if 'spirit' in dis_word:
                confidence += 0.2
            if any_in(['play', 'pensary', 'pense', 'patch', 'port', 'persal', 'perse', 'persion', 'praise'], dis_word):
                confidence -= 0.2

            confidence += sia['neg']
        inflection = getInflection(
            self.chatbot.lp.tokenize(self.dis)[0].lemma_, 'VBD')
        if inflection is None:
            past_participle_form_of_verb = self.dis
        else:
            past_participle_form_of_verb = inflection[0]
        if 'you' in self.normalized:
            response = random_response(DIS_RESPONSES_YOU).format(
                past_participle_form_of_verb)
            emotion = Emotion.angry_non_expressive
        elif 'I' in self.normalized:
            response = "{} {}".format(random_response(
                DIS_RESPONSES_I), random_response(CONSOLATION))
            emotion = Emotion.angel
        else:
            nn = None
            pn = None
            tokenized = spac_token(statement, chatbot=self.chatbot)
            for i in tokenized:
                if (i.pos_ == "NOUN") or (i.pos_ == 'PROPN'):
                    nn = i.text
                elif i.pos_ == 'PRON':
                    pn = i.text
            if not (nn or pn):
                response = 'Lol. What?'
                emotion = Emotion.seriously
            else:
                response = random_response(DIS_RESPONSES_HIM).format(nn or pn)
                emotion = Emotion.cry_overflow
        selected_statement = SugaroidStatement(response, chatbot=True)
        selected_statement.confidence = confidence
        selected_statement.emotion = emotion
        selected_statement.adapter = None
        return selected_statement



import os
import freegames

directory = freegames.__path__[0]

contents = os.listdir(directory)


def game_file(name):
    """"
    Return True if filename represents a game.
    """
    return (
        name.endswith('.py')
        and not name.startswith('__')
        and name != 'utils.py'
    )


games = sorted(name[:-3] for name in contents if game_file(name))



import json
import os

import mistletoe
import wget
from bs4 import BeautifulSoup


class MarkdownReader:
    """
    Converts a Markdown into HTML for parsable output
    """

    def __init__(self, file_path):
        self.file = self.check_file_path(file_path)
        self.html = self.read_markdown(self.file)
        self.soup = self.init_soup(self.html)

        self.headings = []
        self.content = []
        pass

    @staticmethod
    def check_file_path(file_path):
        if os.path.exists(os.path.abspath(file_path)):
            return os.path.abspath(file_path)
        else:
            return FileNotFoundError("The specified file cannot be found")

    def get_headings(self):
        pass

    def get_parsed_content(self):
        headings = []
        content = []
        for i in self.soup.find_all():
            if i.name in ['h1', 'h2', 'h3', 'h4', 'h5', 'h6']:
                headings.append(i.get_text())
                content.append(i.get_text())
            elif i.name in ['p', 'a', 'code']:
                content.append(i.get_text())
        return headings, content

    def get_content(self):
        return self.soup.find_all()

    @staticmethod
    def init_soup(html):
        return BeautifulSoup(html, 'html.parser')

    @staticmethod
    def read(file_path):
        contents = []
        with open(file_path, 'r') as r:
            contents.append(r.read().strip())
        return contents

    @staticmethod
    def read_markdown(path_to_md):
        with open(path_to_md, 'r') as r:
            contents = mistletoe.markdown(r)
        return contents


READ_FILES = dict()

files = {}


def main():
    with open('scrawl.json', 'r') as r:
        files = json.load(r)

    sugar_files = files['sugar']
    try:
        os.makedirs('scrawled')
    except FileExistsError:
        pass
    for i in sugar_files:
        wget.download('https://raw.githubusercontent.com/sugarlabs/{}'
                      .format(i), os.path.join('scrawled', i.split('/')[-1]))
    for markdown_file in os.listdir(os.path.abspath('scrawled')):
        READ_FILES[markdown_file] = MarkdownReader(
            os.path.join('scrawled', markdown_file))

    scrawled = dict()
    for markdown_file in READ_FILES:
        scrawled[markdown_file] = READ_FILES[markdown_file].get_parsed_content()

    with open('scrawled.py', 'w') as w:
        w.write("SCRAWLED = {}".format(scrawled))


if __name__ == '__main__':
    main()



"""
MIT License

Sugaroid Artificial Intelligence
Chatbot Core
Copyright (c) 2020 Srevin Saju

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""

from chatterbot.logic import LogicAdapter
from nltk import word_tokenize, pos_tag

from sugaroid.version import VERSION
from sugaroid.brain.constants import BYE, ANNOYED, DISCLAIMER
from sugaroid.brain.myname import MyNameAdapter
from sugaroid.brain.ooo import Emotion
from sugaroid.brain.postprocessor import random_response
from sugaroid.brain.preprocessors import normalize
from sugaroid.sugaroid import SugaroidStatement


class OneWordAdapter(LogicAdapter):
    """
    Logical adapter for processing data with one words
    """

    def __init__(self, chatbot, **kwargs):
        super().__init__(chatbot, **kwargs)
        self.normalized = None
        self.intersect = None
        self.tokenized = None

    def can_process(self, statement):
        self.normalized = normalize(str(statement).lower())

        if len(self.normalized) == 1:
            return True
        elif len(self.normalized) == 2:
            self.tokenized = pos_tag(self.normalized)
            if self.tokenized[1][1] == ".":
                return True
            else:
                return False
        else:
            return False

    def process(self, statement, additional_response_selection_parameters=None):
        emotion = Emotion.seriously
        confidence = 0.60
        response = random_response(ANNOYED)
        short = str(statement).lower()
        if 'ver' in short:
            response = VERSION
            confidence = 0.99
        elif 'name' in short:
            response = "What name? You should probably use better english"

        elif ('help404' in short) or ('help' in short and '404' in short):
            import sugaroid
            import chatterbot

            help_files = []
            for i in self.chatbot.globals['adapters']:
                help_files.append("{}: {}".format(
                    i.split('.')[-1].strip(), eval(i).__doc__.strip()).strip())
            response = 'hmm. Sure. \n {}'.format('\n '.join(help_files))
            confidence = 0.99

        elif 'help' in short:
            response = 'The help is not very easily provided. ' \
                       'If you are serious of what you are asking, ' \
                       'type help404'
            confidence = 0.99
        elif 'disclaimer' in short:
            response = DISCLAIMER
            confidence = 0.99
        elif 'license' in short:
            lic = """```
                                            MIT License
            
                                  Sugaroid Artificial Intelligence
                                            Chatbot Core
                                   Copyright (c) 2020 Srevin Saju
            
            Permission is hereby granted, free of charge, to any person obtaining a copy
            of this software and associated documentation files (the "Software"), to deal
            in the Software without restriction, including without limitation the rights
            to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
            copies of the Software, and to permit persons to whom the Software is
            furnished to do so, subject to the following conditions:
            
            The above copyright notice and this permission notice shall be included in all
            copies or substantial portions of the Software.
            
            THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
            IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
            FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
            AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
            LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
            OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
            SOFTWARE.```
            """
            response = lic
            confidence = 0.99

        selected_statement = SugaroidStatement(response, chatbot=True)
        selected_statement.confidence = confidence
        selected_statement.emotion = emotion
        selected_statement.adapter = None
        return selected_statement



"""
MIT License

Sugaroid Artificial Intelligence
Chatbot Core
Copyright (c) 2020 Srevin Saju

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.


@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

This module in Sugaroid uses a licensed term 'Akinator'. Sugaroid and its developer
attributes its author here
"
Elokence.com – SARL a limited liability company with a share capital of 80,000€,
whose head office is 8, rue Jules Vallès – 28 300 Mainvilliers – France,
and registered with the Chartres Register of Companies under the number
501 030 316. The design and contents of the Website constitute a protected
work under current intellectual property laws, of which Elokence is the
rightholder. The Publications Director is Mr Arnaud MÉGRET, manager of
Elokence.com.

"

The author takes no right over the information provided by Akinator and is
solely provided by the WebAPI Akinator.com.
All rights reserved to Akinator.

@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
The creator of Akinator.py NinjaSnails is also attributed. The Akinator wrapper
for python is provided under OpenSource License (MIT)


MIT License

Copyright (c) 2019 NinjaSnail1080

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.


"""
import akinator

from sugaroid.brain.postprocessor import random_response

from sugaroid.brain.constants import HOPE_GAME_WAS_GOOD

try:
    from akinator import Akinator, AkiServerDown, AkiTechnicalError, CantGoBackAnyFurther
    akinator_exists = True
except ModuleNotFoundError:
    akinator_exists = False
from chatterbot.logic import LogicAdapter
from sugaroid.sugaroid import SugaroidStatement
from sugaroid.brain.ooo import Emotion
from sugaroid.brain.preprocessors import normalize


AKINATOR_RULES = """
How to Play:
+ think of a character, real or fictional, keep it well in mind and then click on the menu
+ answer the questions as truthfully as possible
+ Sugaroid genie will try to guess your person out
+ Have fun
"""

AKINATOR_ACCEPTED_ANSWER = """
Accepted answers are:
yes: y, yes, 0
no: n, no, 1
probably not: pn, probably not, 4
probably: p, probably, 3
I don't know: idk, i dont know, i don't know, 2
"""


class SugaroidAkinator:
    def __init__(self, chatbot):
        self.chatbot = chatbot
        self.aki = Akinator()
        self.winning = False
        self.check = False
        try:
            self.game_instance = self.aki.start_game("en2")
        except (AkiServerDown, AkiTechnicalError):
            try:
                self.game_instance = self.aki.start_game("en3")
            except (AkiServerDown, AkiTechnicalError):
                self.game_instance = self.aki.start_game("en3")

    def start_game(self):
        # We are about to start the game. Lets send a fascinating entry
        response = "Lets start the play of Akinator™ with me. I, Sugaroid, am your host genie :crystal_ball: for your "\
            "competition{}" \
            "Here is your first question\n{}".format(
                AKINATOR_RULES, self.game_instance)
        self.chatbot.globals['akinator']['enabled'] = True
        return response

    def progression(self, statement):
        if self.aki.progression <= 80:
            user_input = str(statement)
            if (user_input.lower() == "back") or (user_input.lower() == "try again") \
                    or (user_input.lower() == "b"):
                try:
                    self.game_instance = self.aki.back()
                except CantGoBackAnyFurther:
                    pass
            else:
                try:
                    self.game_instance = self.aki.answer(user_input)
                    return self.game_instance
                except akinator.exceptions.InvalidAnswerError:
                    return 'Seems like I cannot understand your answer \n{}'.format(AKINATOR_ACCEPTED_ANSWER)
        else:
            self.winning = True
            return False

    def win(self):
        self.aki.win()
        self.check = True
        return f"It's {self.aki.name} ({self.aki.description})! Was I correct? :thinking_face:\n{self.aki.picture}\n"

    def start_check(self):
        return self.check

    def game_over(self):
        return self.winning

    def check_ans(self, statement):
        statement = str(statement)
        if ('yes' in statement.lower()) or ('yea' in statement.lower()) or ('exactly' in statement.lower()) or \
                ('obviously' in statement.lower()) or ('correct' in statement.lower()) or \
                ('right' in statement.lower()) or ('you' in statement.lower() and 'won' in statement.lower()) or \
                statement.lower() == "yes" or statement.lower() == "y" or statement.lower() == "yea":
            response = "Yay! I won the game! :punch: :jack_o_lantern: :gift: :tada:"
        else:
            response = "Oh. I failed the same. Seems like you are smarter than me. :weary:"
        response = response + \
            "\n{}\n I am back to my business".format(
                random_response(HOPE_GAME_WAS_GOOD))
        self.chatbot.globals['akinator']['enabled'] = False
        self.chatbot.globals['akinator']['class'] = None
        return response


class AkinatorAdapter(LogicAdapter):
    """
    Adapter which ports the wrapper of the Akinator game to Sugaroid
    """

    def __init__(self, chatbot, **kwargs):
        super().__init__(chatbot, **kwargs)
        self.aki = False
        self.game_instance = None
        self.normalized = None

    def can_process(self, statement):
        self.normalized = normalize(str(statement).lower())
        if (('akinator' in self.normalized) and akinator_exists) and ('not' not in self.normalized):
            return True
        else:
            return self.chatbot.globals['akinator']['enabled']

    def process(self, statement, additional_response_selection_parameters=None):
        response = None
        confidence = 2.0  # FIXME: Override all other answers
        emotion = Emotion.genie
        selected_statement = SugaroidStatement(
            "Akinator's servers are temporarily down. I am sorry for the "
            "inconvenience.",
           chatbot=True
        )
        selected_statement.confidence = confidence
        selected_statement.emotion = emotion
        return selected_statement

        if 'stop' in self.normalized:
            self.chatbot.globals['akinator']['enabled'] = False
            response = 'I am sorry. You quit the game abrubtly. {}'.format(
                random_response(HOPE_GAME_WAS_GOOD))
        elif not self.chatbot.globals['akinator']['enabled']:
            self.chatbot.globals['akinator']['class'] = SugaroidAkinator(
                self.chatbot)
            response = self.chatbot.globals['akinator']['class'].start_game()
        else:
            if not self.chatbot.globals['akinator']['class'].game_over():
                response = \
                    self.chatbot.globals['akinator']['class'].progression(
                        statement
                    )
                if not response:
                    response = self.chatbot.globals['akinator']['class'].win()
            else:
                if self.chatbot.globals['akinator']['class'].start_check():
                    response = \
                        self.chatbot.globals['akinator']['class'].check_ans(
                            statement
                        )

        selected_statement = SugaroidStatement(response, chatbot=True)
        selected_statement.confidence = confidence
        selected_statement.emotion = emotion
        return selected_statement



"""
MIT License

Sugaroid Artificial Intelligence
Chatbot Core
Copyright (c) 2020 Srevin Saju

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""

import nltk
from chatterbot.logic import LogicAdapter
from sugaroid.brain.ooo import Emotion
from sugaroid.brain.preprocessors import normalize
from sugaroid.brain.rereversei import reset_reverse
from sugaroid.sugaroid import SugaroidStatement


class BoolAdapter(LogicAdapter):
    """
    Processes Boolean based answers
    """

    def __init__(self, chatbot, **kwargs):
        super().__init__(chatbot, **kwargs)
        self.chatbot = chatbot
        self.normalized = None
        self.intersect = None
        self.normalized = None
        self.tagged = None
        self.last_normalized = None
        self.bool = None

    def can_process(self, statement):
        self.normalized = normalize(str(statement).lower())
        if ('yes' in self.normalized) or ('no' in self.normalized) or ('true' in self.normalized) or ('false' in self.normalized):
            if ('yes' in self.normalized) or ('true' in self.normalized):
                self.bool = True
            else:
                self.bool = False
            return True
        else:
            return False

    def process(self, statement, additional_response_selection_parameters=None):
        bool_yes = 'yes' in self.normalized
        emotion = Emotion.neutral
        if self.chatbot.report:
            if bool_yes:
                response = 'Sure, I would connect to the Developer to report this issue right away'
                self.chatbot.report = False
                # TODO: Add report function
                # report_here() FIXME
            else:
                response = 'Ok, I will not report it.'
                self.chatbot.report = False
            confidence = 1.0
        elif self.chatbot.globals['trivia_answer']:
            if self.chatbot.globals['trivia_answer'] == self.bool:
                response = "Exactly! You are right"
                reset_reverse(self)
            else:
                response = 'Nope, You got it wrong. The correct answer was {}'.format(
                    self.chatbot.globals['trivia_answer'])
                reset_reverse(self)
            self.chatbot.globals['trivia_answer'] = None
            confidence = 1.1
        else:
            if self.chatbot.globals['history']['total'][-1] == 0:
                if bool_yes:
                    response = 'I shall annoy you. A big NO'
                else:
                    response = 'I would rather have fun, YES ?'
            else:
                md = False
                vb = False
                nn = False
                self.last_normalized = normalize(
                    str(self.chatbot.globals['history']['total'][-1]))
                self.tagged = nltk.pos_tag(self.last_normalized)
                iteration = 0
                for j in self.tagged:
                    if j[1].startswith('MD'):
                        md = True
                    elif (j[1].startswith('VB')) and (not j[0] == 'be'):
                        vb = True
                        verb = j[0]
                    elif j[1].startswith('NN'):
                        if not nn:
                            nn_index = iteration
                        nn = True
                        noun = j[0]
                    iteration += 1

                if md:
                    if nn:
                        some_nouns = ' '.join(self.last_normalized[nn_index:])
                    if bool_yes:
                        if nn:
                            response = 'Ok, here comes your {} 😝😝'.format(
                                some_nouns)
                        elif vb:
                            response = 'You should {}'.format(
                                verb.replace('ing', ''))
                        else:
                            response = 'I will keep thinking 🚀'
                    else:
                        if nn:
                            response = 'Ok, I will have the {}'.format(
                                some_nouns)
                        elif vb:
                            response = "You shouldn't {} then".format(
                                verb.replace('ing', ''))
                        else:
                            response = 'Okay!'
                else:
                    if bool_yes:
                        response = "Why is this 'yes' here? I couldn't find the question. Anyway, I agree with you"
                    else:
                        response = 'No? for what?.'
                        emotion = Emotion.angry

            confidence = 0.95
        selected_statement = SugaroidStatement(response, chatbot=True)
        selected_statement.confidence = confidence

        selected_statement.emotion = emotion

        return selected_statement



"""
MIT License

Sugaroid Artificial Inteligence
Chatbot Core
Copyright (c) 2020 Srevin Saju

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""
import logging

from chatterbot.logic import LogicAdapter
from chatterbot.trainers import ListTrainer
from nltk import word_tokenize

from sugaroid.sugaroid import SugaroidStatement
from sugaroid.brain.ooo import Emotion
from sugaroid.brain.preprocessors import normalize


class LearnAdapter(LogicAdapter):
    """
    a specific adapter for learning responses
    """

    def __init__(self, chatbot, **kwargs):
        super().__init__(chatbot, **kwargs)

    def can_process(self, statement):
        normalized = word_tokenize(str(statement).lower())
        try:
            last_type = self.chatbot.globals['history']['types'][-1]
        except IndexError:
            last_type = False
        logging.info(
            'LearnAdapter: can_process() last_adapter was {}'.format(last_type))

        if 'learn' in normalized and 'not' not in normalized and not 'to' in normalized:
            return True
        elif self.chatbot.globals['learn'] and (last_type == 'LearnAdapter'):
            return True
        else:
            if self.chatbot.globals['learn']:
                self.chatbot.globals['learn'] = False
            return False

    def process(self, statement, additional_response_selection_parameters=None):
        response = None
        if not self.chatbot.globals['learn']:
            response = 'Enter something you want to teach me. What is the statement that you want me to learn.'
            self.chatbot.globals['learn'] = 2
        elif self.chatbot.globals['learn'] == 2:
            response = 'What should I respond to the above statement?'
            self.chatbot.globals['learn_last_conversation'].append(
                str(statement))
            self.chatbot.globals['learn'] -= 1
        elif self.chatbot.globals['learn'] == 1:
            response = 'Thanks for teaching me something new. I will always try to remember that'
            self.chatbot.globals['learn_last_conversation'].append(
                str(statement))
            self.chatbot.globals['learn'] -= 1
            list_trainer = ListTrainer(self.chatbot)
            list_trainer.train(self.chatbot.globals['learn_last_conversation'])

        selected_statement = SugaroidStatement(response, chatbot=True)
        selected_statement.confidence = 9
        selected_statement.adapter = 'LearnAdapter'
        emotion = Emotion.lol
        selected_statement.emotion = emotion
        return selected_statement



"""
MIT License

Sugaroid Artificial Intelligence
Chatbot Core
Copyright (c) 2020 Srevin Saju

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

News headlines provided by NewsAPI
"""
import logging
import os

from chatterbot.logic import LogicAdapter
from newsapi import NewsApiClient
from dotenv import load_dotenv
from sugaroid.brain.ooo import Emotion
from sugaroid.brain.preprocessors import spac_token

from sugaroid.sugaroid import SugaroidStatement

NEWSAPI_COUNTRIES = {
    'argentina': 'ar', 'australia': 'au', 'austria': 'at', 'belgium': 'be', 'brazil': 'br',
    'bulgaria': 'bg', 'canada': 'ca', 'china': 'cn', 'colombia': 'co', 'cuba': 'cu',
    'czech republic': 'cz', 'egypt': 'eg', 'france': 'fr', 'germany': 'de', 'greece': 'gr',
    'hong kong': 'hk', 'hungary': 'hu', 'india': 'in', 'indonesia': 'id', 'ireland': 'ie',
    'israel': 'il', 'italy': 'it', 'japan': 'jp', 'latvia': 'lv', 'lithuania': 'lt',
    'malaysia': 'my', 'mexico': 'mx', 'morocco': 'ma', 'netherlands': 'nl', 'new zealand': 'nz',
    'nigeria': 'ng', 'norway': 'no', 'philippines': 'ph', 'poland': 'pl', 'portugal': 'pt',
    'romania': 'ro', 'russia': 'ru', 'saudi arabia': 'sa', 'serbia': 'rs', 'singapore': 'sg',
    'slovakia': 'sk', 'slovenia': 'si', 'south africa': 'za', 'south korea': 'kr', 'sweden': 'se',
    'switzerland': 'ch', 'taiwan': 'tw', 'thailand': 'th', 'turkey': 'tr', 'uae': 'ae',
    'ukraine': 'ua', 'united kingdom': 'gb', 'united states': 'us', 'venuzuela': 've',
    'us': 'us', 'uk': 'uk', 'u.k.': 'uk', 'u.s.': 'us', 'u.s': 'us', 'u.k': 'uk', 'u s': 'us', 'u k': 'uk',
    'the united states': 'us', 'the united kingdom': 'uk'
}


class SugaroidNews:
    """
    A dedicated class for processing news from the API
    """

    def __init__(self, chatbot=None):
        load_dotenv()
        self.chatbot = chatbot
        token = os.getenv('NEWSAPI_TOKEN')
        self.api = NewsApiClient(api_key=token)

    def get_top_headlines(self, country=None):
        """
        Get the top news items of the country if provided. Otherwise give international news
        :param country:
        :return:
        """
        try:
            if country is None:
                response = self.api.get_top_headlines()
            else:
                response = self.api.get_top_headlines(country=country)
            return self.process(response)
        except Exception as e:
            if str(e) == 'invalid country':
                self.chatbot.globals['last_news'] = False
                logging.info(
                    f'SugaroidNews: Failed to get the news. country=[{type(country)}, {country}]')
                return ['The country you provided does not exist in my database. I am sorry.']
            self.chatbot.globals['last_news'] = False
            return ['Failed to establish connection with the server. Error: {}'.format(e)]

    def get_news_keyword(self, keyword):
        try:
            response = self.api.get_everything(q=keyword)
            return self.process(response)
        except Exception as e:
            self.chatbot.globals['last_news'] = False
            return ['Failed to establish connection with the server. Error: {}'.format(e)]

    def process(self, response):
        """
        Convert the raw JSON Response to Human Readable News bullets
        :param response:
        :return: array of news headlines
        """

        news_headlines = []
        for i in response['articles']:
            try:
                news_headlines.append("{} from {}".format(
                    i['title'], i['source']['name']))
            except Exception as e:
                pass
        if not news_headlines:
            self.chatbot.globals['last_news'] = False
            return ['There was an error collecting the latest news headlines. Please try again later.']
        self.chatbot.globals['last_news'] = response
        return news_headlines


class NewsAdapter(LogicAdapter):
    """
    Ports the SugaroidNews Wrapper for easier access by the SugaroidChatbot Class
    """

    def __init__(self, chatbot, **kwargs):
        super().__init__(chatbot, **kwargs)
        self.cos = None
        self.integer = False

    def can_process(self, statement):
        """
        Checks if the statement has a cosine similarity of any of the words mentioned here
        :param statement: SugaroidStatement Class Object
        :return: bool,
        """
        _ = self.chatbot.lp.similarity
        try:
            last_type = self.chatbot.globals['history']['types'][-1]
        except IndexError:
            last_type = False

        logging.info(
            'NewsAdapter: can_process() last_adapter was {}'.format(last_type))
        processed = str(statement).replace('?', '').replace('.', '')
        self.cos = max([
            _(processed, 'Whats happening'),
            _(processed, 'Get me the latest headlines'),
            _(processed, 'Show me today\'s news'),
            _(processed, 'Can you tell me todays news'),
            _(processed, 'Can you show me today\'s headlines'),
            _(processed, 'What is happening today'),
            _(processed, 'What top stories running right now')

        ])
        logging.info(
            "NewsAdapter received a cosine similarity of {}".format(self.cos))
        if self.cos > 0.8 or 'news' in processed.lower() or 'headlines' in processed.lower():
            return True
        elif 'show me more' in str(statement).lower() and self.chatbot.globals['last_news']:
            for i in spac_token(statement, self.chatbot):
                if i.tag_ in ['LS', 'CD']:
                    self.integer = i.lower_
                    return True
            else:
                return False
        elif self.chatbot.globals['last_news'] and (last_type == 'NewsAdapter'):
            for i in spac_token(statement, self.chatbot):
                if i.tag_ in ['LS', 'CD']:
                    self.integer = i.lower_
                    return True
            else:
                return False
        else:
            return False

    def process(self, statement, additional_response_selection_parameters=None):

        confidence = self.cos
        country = None
        keyword = False
        # Override the other answer when asked for
        normalize = spac_token(statement, self.chatbot)
        if len(normalize) <= 2:
            confidence = confidence + 0.5
        if self.integer and self.chatbot.last_news:
            response = "Your selected integer {}, response {}".format(
                self.integer, self.chatbot.last_news)
            try:
                self.integer = int(self.integer)
            except ValueError:
                response = 'Sorry, I couldn\'t make out what number it actually is. I might understand it better if ' \
                           'it is in a numerical form? '
            if type(self.integer) is int:
                try:
                    data = self.chatbot.last_news['articles'][self.integer - 1]
                    provider = data['source']['name']
                    author = data['author']
                    last_updated = data['publishedAt']
                    title = data['title']
                    description = data['description']
                    url = data['url']
                    img_url = data['urlToImage']
                    response = f"From {provider}\nWritten by {author} on {last_updated}\n\n{title}\n\n{description}" \
                               f"\nRead more at {url}\n{img_url} "

                except IndexError:
                    response = "I am sorry. I couldn't find the item you actually asked for."
        else:
            for ent in normalize.ents:
                if ent.label_ == 'GPE':
                    country = ent
                elif ent.label_ in ['PERSON', 'NORP', 'FAC', 'ORG', 'LOC', 'PRODUCT', 'EVENT', 'WORK_OF_ART', 'LAW']:
                    keyword = ent
            logging.info("NewsAdapter found as Natural Entity Recognizer words, country : {}, keyword: {}"
                         .format(country, keyword))
            for i in range(len(normalize)-1):
                if normalize[i].lower_ == "about" or normalize[i].lower_ == 'on':
                    logging.info("NewsAdapter: found determiner about or on")
                    if not keyword:
                        keyword_words = []
                        for j in range(i+1, len(normalize)):
                            if normalize[j].pos_ == 'DET':
                                continue
                            else:
                                keyword_words.append(normalize[j].text)

                        keyword = ' '.join(keyword_words)
            sg_news = SugaroidNews(self.chatbot)
            if keyword:
                logging.info(
                    "NewsAdapter: Getting news of keyword {}".format(keyword))
                news = sg_news.get_news_keyword(str(keyword))
            else:
                logging.info(
                    "NewsAdapter: Getting news of country {}".format(country))
                news = sg_news.get_top_headlines(
                    country=NEWSAPI_COUNTRIES.get(str(country).lower(), None))

            def bracketize(x):
                return '\n:large_blue_diamond: [{}] {}'.format(x[0] + 1, str(x[1]))

            response = ' '.join([bracketize(x) for x in enumerate(news)])

        selected_statement = SugaroidStatement(response, chatbot=True)
        selected_statement.confidence = confidence
        emotion = Emotion.neutral
        selected_statement.adapter = 'NewsAdapter'
        selected_statement.emotion = emotion

        return selected_statement



from sugaroid.brain.ooo import Emotion

from sugaroid.brain.constants import WHAT_I_AM_GOING_TO_DO
from sugaroid.sugaroid import SugaroidStatement


def process_what_ami_doing(statement: SugaroidStatement):
    response = None
    for i in statement.doc:
        if i.lower_ in WHAT_I_AM_GOING_TO_DO.keys():
            response = WHAT_I_AM_GOING_TO_DO[i.lower_]
            break
    if response:
        st = SugaroidStatement(response, confidence=0.8,
                               chatbot=True, emotion=Emotion.cry_overflow)
        st.confidence = 0.8
        return st
    else:
        return SugaroidStatement("Ok", confidence=0, chatbot=True, emotion=Emotion.cry_overflow)



"""
MIT License

Sugaroid Artificial Intelligence
Chatbot Core
Copyright (c) 2020 Srevin Saju

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""

from chatterbot.logic import LogicAdapter
import logging

from sugaroid.brain.ooo import Emotion
from sugaroid.brain.postprocessor import random_response, any_in
from sugaroid.brain.preprocessors import normalize
from sugaroid.sugaroid import SugaroidStatement


ABOUT_CORONAVIRUS = """
Coronavirus disease (COVID-19) is an infectious disease caused by a new virus.
The disease causes respiratory illness (like the flu) with symptoms such as a 
cough, fever, and in more severe cases, difficulty breathing. You can protect 
yourself by washing your hands frequently, avoiding touching your face, and 
avoiding close contact (1 meter or 3 feet) with people who are unwell.

Lets stay safe during covid 19 by staying at home. 
I can do an approximation if you do have coronavirus.
"""

COVID_QUESTIONS = [
    [1, 'Do you have fever?', 1],
    [2, 'Do you have cough?', 1],
    [3, 'Do you have shortness of breath', 2],
    [4, 'Have you travelled outside your house in the last 14 days?', 1],
    [5, 'Have you been tested for covid before, with negative results?', 0.1],
    [6, 'Do you have any chronic diseases, eg: asthma or faced pneumonia in the past', 1],
    [7, '{}', 0]
]


class CovidAdapter(LogicAdapter):
    """
    A COVID-19 dedicated Adapter
    In the **memory** of cancelled Google Code In, Grand Prize Winner's trip, 2019-2020 :
    """
    """
    "Nor Mars his sword nor war’s quick fire shall burn
    The living record of (GCI GPW-19's) memory.
    ’Gainst death and all-oblivious enmity
    Shall you pace forth; your praise shall still find room
    Even in the eyes of all posterity (future GPWs)
    That wear this world out to the ending doom.
    So, till the Judgement that yourself arise,
    You (GCI GPW 2019) live in this, and dwell in lovers’ (coder's) eyes."
            ~ William Shakespeare (Sonnet 55)
    """

    def __init__(self, chatbot, **kwargs):
        super().__init__(chatbot, **kwargs)
        self.normalized = None
        self.chatbot = chatbot

    def can_process(self, statement):
        self.normalized = normalize(str(statement))
        if any_in(['covid', 'covid19', 'covid-19', 'corona', 'coronavirus'], self.normalized):
            return True
        else:
            return False

    def process(self, statement, additional_response_selection_parameters=None):
        confidence = 1
        # FIXME Creates unusual response
        response = ABOUT_CORONAVIRUS

        if any_in(['I', 'i'], self.normalized):
            response = "I will do a short approximation if you do have coronavirus\n{covidq}"\
                .format(covidq=COVID_QUESTIONS[0][1])
            self.chatbot.globals['reversei']['uid'] = 'CORONAVIRUS'
            self.chatbot.globals['reversei']['enabled'] = True
            logging.info(
                f"CovidAdapter sets ['reversei']['enabled'] as {self.chatbot.globals['reversei']['enabled']}")
            self.chatbot.globals['reversei']['data'] = [1, 0]

        elif 'you' in self.normalized:
            response = 'Someone told that I had been contracted with corona from somewhere, but thats extremely wrong.'\
                       ' I will not get infected by any physical virus, (except Trojan or NO_HEROKU_CREDIT virus)'

        selected_statement = SugaroidStatement(response, chatbot=True)
        selected_statement.confidence = confidence
        emotion = Emotion.neutral
        selected_statement.emotion = emotion

        return selected_statement



"""
MIT License

Sugaroid Artificial Inteligence
Chatbot Core
Copyright (c) 2020 Srevin Saju

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""


from chatterbot.logic import LogicAdapter
from nltk import word_tokenize

from sugaroid.brain.constants import EMOJI_SMILE, FUN_ASK_QUESTION, FUN_LET_ME_TRY
from sugaroid.brain.ooo import Emotion
from sugaroid.brain.postprocessor import random_response, reverse
from sugaroid.brain.preprocessors import spac_token
from sugaroid.sugaroid import SugaroidStatement


class FunAdapter(LogicAdapter):
    """
    Gives a random response, because Sugaroid tries not to say I don't know
    """

    def __init__(self, chatbot, **kwargs):
        super().__init__(chatbot, **kwargs)

    def can_process(self, statement):
        if self.chatbot.globals['fun']:
            return True
        else:
            return False

    def process(self, statement, additional_response_selection_parameters=None):
        emotion = Emotion.neutral
        confidence = 0.05
        parsed = str(statement)
        if 'not' in parsed:
            # if you are not, who then is?
            suffix = " either. "
            prefix = ""
            emotion = Emotion.wink
            confidence *= 2
        else:
            interrogation = False
            token = spac_token(statement, chatbot=self.chatbot)
            for i in token:
                # checks if the statement contains any sequence of interrogative type of words
                if i.tag_ == '.' and i.text == '?':
                    interrogation = True
                if str(i.tag_).startswith('W'):
                    interrogation = True

            if interrogation:
                prefix, suffix = '', ''
                confidence *= 2
                parsed = random_response(FUN_ASK_QUESTION).format(
                    ' '.join(reverse(word_tokenize(str(statement)))))  # This seems complex.
                # The tokenized input statement is reversed using the reverse unction
                # Reverse in this sense means switching first person and second person nouns
                # The returned list of tokens are then converted into a string by joining each element
                # to a whitespace making a sentence, which is then converted to lower case
                # for the visibility sake
            else:
                prefix, suffix = random_response(FUN_LET_ME_TRY)
                suffix = "' {}".format(suffix.format(
                    random_response(EMOJI_SMILE)))
                prefix = "{} '".format(prefix)
                emotion = Emotion.wink

        selected_statement = SugaroidStatement("{pre}{main}{fix}".format(
            pre=prefix, main=parsed, fix=suffix), chatbot=True)
        selected_statement.confidence = confidence
        selected_statement.emotion = emotion
        return selected_statement



"""
MIT License

Sugaroid Artificial Inteligence
Chatbot Core
Copyright (c) 2020 Srevin Saju

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""
import logging
from random import randint
from nltk.corpus import stopwords
import nltk
from sugaroid.brain.utils import LanguageProcessor


def sigmaSimilarity(src, dest):
    """
    :param src: a list of literals
    :param dest: a list of final literals
    :return: Probability
    """
    total = len(src)
    sum = 0
    for i in src:
        for j in dest:
            if i == j:
                sum += 1
    return sum/total


def difference(lst1, lst2):
    lst3 = [value for value in lst1 if value not in lst2]
    return lst3


def reverse(token):
    """
    Reverses the first person pronouns to second person pronouns and vice versa
    :param token: a nltk.word_tokenize type list
    :return: a list similar to nltk.word_tokenize
    """
    processed = []
    has_am = 'am' in token
    has_is = 'are' in token
    logging.info("Reverse: Received {}".format(token))
    interrogation = False
    for i in token:
        lps = LanguageProcessor().tokenize(i)[0]
        if lps.tag_ == '.' and lps.lower_ == '?':
            interrogation = True
        elif str(lps.tag_).startswith('W'):
            interrogation = True
    for i in token:
        tagged = nltk.pos_tag([i])
        if tagged[0][1] == 'PRP':
            if i == 'you':
                if interrogation:
                    processed.append('I')
                else:
                    processed = processed[:-1] + ['I'] + processed[-1:]
            elif i.lower() == 'i':
                processed.append('you')
        elif tagged[0][1] == 'VBP':
            if i == 'are':
                if 'I' in processed:
                    processed.append('am')
                else:
                    processed.append('are')
            elif i == 'am':
                if 'I' in processed:
                    processed.append('am')
                else:
                    processed.append('are')
            else:
                processed.append('are')
        else:
            processed.append(i)

    for j in range(0, len(processed)-2):
        if processed[j] == 'I' and processed[j+1] == 'are':
            processed[j+1] = 'am'
        elif processed[j] == 'you' and processed[j+1] == 'am':
            processed[j + 1] = 'are'
        else:
            continue

    for j in range(0, len(processed)-2):
        if processed[j] == 'I' and processed[j+1] == 'are':
            processed[j] = 'you'
        elif processed[j] == 'you' and processed[j+1] == 'am':
            processed[j] = 'I'
        else:
            continue

    for j in range(0, len(processed)-2):
        if processed[j] == 'are' and processed[j+1] == 'I':
            processed[j] = 'am'
        elif processed[j] == 'am' and processed[j+1] == 'you':
            processed[j] = 'are'
        else:
            continue

    logging.info("Reverse: Pushing {}".format(processed))
    return processed


def random_response(iterable=()):
    """
    Selects a random response from the given set of iterable types
    :param iterable:
    :return: a selected value of the iterable
    """
    return iterable[randint(0, len(iterable)-1)]


def cosine_similarity(X_list, Y_list):

    # Program to measure similarity between
    # two sentences using cosine similarity.

    # sw contains the list of stopwords
    sw = stopwords.words('english')
    l1 = []
    l2 = []

    # remove stop words from string
    X_set = {w for w in X_list if not w in sw}
    Y_set = {w for w in Y_list if not w in sw}

    # form a set containing keywords of both strings
    rvector = X_set.union(Y_set)
    for w in rvector:
        if w in X_set:
            l1.append(1)  # create a vector
        else:
            l1.append(0)
        if w in Y_set:
            l2.append(1)
        else:
            l2.append(0)
    c = 0

    # cosine formula
    for i in range(len(rvector)):
        c += l1[i] * l2[i]
    cosine = c / float((sum(l1) * sum(l2)) ** 0.5)
    return cosine


def any_in(arg1: list, string1: list):
    """
    Advanced in operator
    Checks any of the list item exists in in the string
    :param arg1:
    :param string1:
    :return:
    """
    for i in arg1:
        if i in string1:
            return True
    else:
        return False


def raw_in(arg, spacy_tokenized):
    for i in spacy_tokenized:
        if i.text == arg:
            return True
    else:
        return False


def raw_lower_in(arg, spacy_tokenized):
    for i in spacy_tokenized:
        if i.lower_ == arg:
            return True
    else:
        return False


def lemma_in(arg, spacy_tokenized):
    for i in spacy_tokenized:
        if i.lemma_ == arg:
            return True
    else:
        return False


def pos_in(arg, spacy_tokenized):
    for i in spacy_tokenized:
        if i.pos_ == arg:
            return True
    else:
        return False


def text2int(textnum, numwords={}):
    """
    Converts words to numbers
    from https://stackoverflow.com/questions/493174/is-there-a-way-to-convert-number-words-to-integers
    :param textnum:
    :param numwords:
    :return:
    """
    if not numwords:
        units = [
            "zero", "one", "two", "three", "four", "five", "six", "seven", "eight",
            "nine", "ten", "eleven", "twelve", "thirteen", "fourteen", "fifteen",
            "sixteen", "seventeen", "eighteen", "nineteen",
        ]

        tens = ["", "", "twenty", "thirty", "forty",
                "fifty", "sixty", "seventy", "eighty", "ninety"]

        scales = ["hundred", "thousand", "million", "billion", "trillion"]

        numwords["and"] = (1, 0)
        for idx, word in enumerate(units):
            numwords[word] = (1, idx)
        for idx, word in enumerate(tens):
            numwords[word] = (1, idx * 10)
        for idx, word in enumerate(scales):
            numwords[word] = (10 ** (idx * 3 or 2), 0)

    current = result = 0
    for word in textnum.split():
        if word not in numwords:
            raise Exception("Illegal word: " + word)

        scale, increment = numwords[word]
        current = current * scale + increment
        if scale > 100:
            result += current
            current = 0

    return result + current



"""
MIT License

Sugaroid Artificial Intelligence
Chatbot Core
Copyright (c) 2020 Srevin Saju

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""

import logging
import spacy
from chatterbot.logic import LogicAdapter
from nltk.sentiment import SentimentIntensityAnalyzer
from sugaroid.brain.constants import GREET, BURN_IDK, I_AM
from sugaroid.brain.ooo import Emotion
from sugaroid.brain.postprocessor import cosine_similarity, random_response, raw_in, raw_lower_in
from sugaroid.brain.whatamidoing import process_what_ami_doing
from sugaroid.sugaroid import SugaroidStatement


class MeAdapter(LogicAdapter):
    """
    Processes the statements showing possessive
    """

    def __init__(self, chatbot, **kwargs):
        super().__init__(chatbot, **kwargs)

        self.normalized = None
        self.tokenized = None
        self.nlp = spacy.load("en_core_web_sm")

    def can_process(self, statement):
        # TODO Fix this
        self.tokenized = self.nlp(str(statement))

        for i in range(len(self.tokenized) - 1):
            if self.tokenized[i].pos_ == 'PRON' and str(self.tokenized[i + 1].tag_).startswith('VB'):
                return True
        else:
            return False

    def process(self, statement, additional_response_selection_parameters=None):
        response = 'ok'
        confidence = 0
        emotion = Emotion.neutral
        if raw_in('I', self.tokenized):
            logging.info(str(["{} {} {}".format(k, k.tag_, k.pos_)
                              for k in self.tokenized]))
            # check if the pronoun has been reached yet, otherwise may detect some other nouns
            start_scanning = False
            for i in self.tokenized:
                if i.pos_ == 'PRON' and not i.tag_.startswith("W"):
                    start_scanning = True
                if start_scanning:
                    pass
                elif (i.pos_ != 'PRON') or (not start_scanning) or (i.tag_.startswith('W')):
                    logging.info(
                        "MeAdapter: Skipping {} {}".format(i.lower_, i.tag_))
                    continue
                logging.info(
                    "MeAdapter: Scanning :: {} : {}".format(i.text, i.pos_))
                if (i.pos_ == 'PROPN') or (i.tag_ == 'NN'):
                    nn = i.text
                    if self.chatbot.globals['USERNAME']:
                        response = "Are you sure you are {n}? I thought you were {u}".format(
                            n=nn, u=self.chatbot.globals['USERNAME'])
                        emotion = Emotion.wink
                        if i.pos_ == 'PROPN':
                            confidence = 0.95
                        else:
                            confidence = 0.8
                        self.chatbot.globals['nn'] = nn
                        self.chatbot.globals['reversei']['uid'] = 30000000001
                        self.chatbot.globals['reversei']['type'] = bool
                        self.chatbot.globals['reversei']['enabled'] = True
                        emotion = Emotion.non_expressive_left
                        break
                    else:
                        if not ('not' in str(statement)):
                            if i.lower_ == 'sugaroid':
                                response = random_response(I_AM)
                                emotion = Emotion.lol
                                confidence = 0.95
                            else:
                                response = random_response(
                                    GREET).format(str(nn).capitalize())
                                confidence = 0.9
                                self.chatbot.globals['USERNAME'] = nn
                                emotion = Emotion.positive
                            break
                        else:
                            response = 'Ok!'
                            confidence = 0.5
                            emotion = Emotion.seriously
                            break
                elif i.lower_ == 'sugaroid':
                    response = random_response(I_AM)
                    emotion = Emotion.lol
                    confidence = 0.95
                else:
                    sia = SentimentIntensityAnalyzer()
                    ps = sia.polarity_scores(str(i.sent))
                    confidence = 0.35
                    if ps['neu'] == 1:
                        response = 'Ok! Thats great to hear from you'
                        emotion = Emotion.lol
                    elif ps['pos'] > ps['neg']:
                        response = 'Yay! I agree to you'
                        emotion = Emotion.positive
                    else:
                        confidence = 0.2
                        response = 'Think again'
                        emotion = Emotion.non_expressive_left
        elif raw_lower_in('you', self.tokenized):
            logging.info(str(["{} {} {}".format(k, k.tag_, k.pos_)
                              for k in self.tokenized]))
            nn = ''
            start_scanning = False
            for i in self.tokenized:
                if i.pos_ == 'PRON' and not i.tag_.startswith("W"):
                    start_scanning = True
                if start_scanning:
                    pass
                elif (i.pos_ != 'PRON') or (not start_scanning) or (i.tag_.startswith('W')):
                    logging.info("MeAdapter: Skipping {} {} ss={} {}"
                                 .format(i.lower_, i.tag_, not start_scanning, i.pos_))
                    continue
                logging.info(
                    "MeAdapter: Scanning :: {} : {}".format(i.text, i.pos_))
                if i.pos_ == 'ADJ':

                    cos = cosine_similarity([str(i.lower_)], ['sugaroid'])
                    if i.lower_ == 'sugaroid':
                        nn = i.text
                        response = "Yup, that's my name. I am sugaroid"
                        emotion = Emotion.lol
                        confidence = 0.9
                        break
                    elif cos > 0.9:
                        response = "Yes, you were close! My name is sugaroid"
                        emotion = Emotion.positive
                        confidence = 0.9
                        break
                    else:
                        if i.lower_ in ['human', 'animal', 'bird']:
                            response = 'No, I am not a {adj}. I am a robot'.format(
                                adj=i.lower_)
                            emotion = Emotion.angry_non_expressive
                            confidence = 0.9
                        else:
                            response = 'seriously?'
                            emotion = Emotion.angry
                            confidence = 0.09

                elif i.pos_ == 'PROPN':
                    cos = cosine_similarity([str(i.lower_)], ['sugaroid'])

                    if i.lower_ == 'sugaroid':
                        nn = i.text
                        response = "Yup, that's my name. I am sugaroid"
                        emotion = Emotion.lol
                        confidence = 0.9
                        break
                    elif cos > 0.9:
                        response = "Yes, you were close! My name is sugaroid"
                        emotion = Emotion.positive
                        confidence = 0.9
                        break
                    else:

                        nn = i.text
                        response = "Nope, I am not {n}, I am sugaroid".format(
                            n=nn)
                        emotion = Emotion.angry
                        confidence = 0.9

                elif i.tag_ == 'NN':
                    if i.lower_ in ['bot', 'robot', 'computer', 'silicon', 'infant']:
                        response = 'You are right! I am a {}'.format(i.lower_)
                        confidence = 0.9
                        emotion = Emotion.positive
                    elif i.lower_ in ['human', 'bird', 'animal', 'tree', 'politician', 'player', 'liar', 'priest']:
                        response = 'No way! I can\'t imagine myself to be a {}'.format(
                            i.lower_)
                        confidence = 0.9
                        emotion = Emotion.vomit
                    else:
                        logging.info(
                            "MeAdapter: Couldn't classify type of noun {}".format(i.lower_))
                        confidence = 0.9
                        sia = SentimentIntensityAnalyzer()
                        ps = sia.polarity_scores(str(i.sent))
                        if ps['neu'] == 1.0:
                            # try presence adapter pieces
                            presence_statement = process_what_ami_doing(
                                statement)
                            if presence_statement.confidence == 0:

                                response = 'I will need more time to learn if that actually makes sense with respect to ' \
                                           'myself. '
                                emotion = Emotion.cry
                            else:
                                return presence_statement
                        elif ps['pos'] > ps['neg']:
                            response = 'I guess I am {}. Thanks!'.format(
                                i.text)
                            emotion = Emotion.wink
                        else:
                            response = 'I am not {}! I am Sugaroid.'.format(
                                i.lower_)
                            emotion = Emotion.angry

                elif i.tag_.startswith('VB'):
                    root_verb = i.lemma_
                    if root_verb in ['say', 'tell', 'speak', 'murmur', 'blabber', 'flirt']:
                        response = random_response(BURN_IDK)
                        emotion = Emotion.lol
                        confidence = 0.8

        else:
            presence_statement = process_what_ami_doing(statement)
            if presence_statement.confidence == 0:
                # FIXME : Add more logic here
                response = 'Ok'
                confidence = 0.3
                emotion = Emotion.non_expressive_left
            else:
                return presence_statement

        selected_statement = SugaroidStatement(response, chatbot=True)
        selected_statement.confidence = confidence
        selected_statement.emotion = emotion
        return selected_statement



"""
MIT License

Sugaroid Artificial Intelligence
Chatbot Core
Copyright (c) 2020 Srevin Saju

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""


import random
import sys

from chatterbot.conversation import Statement
from chatterbot.logic import LogicAdapter

from sugaroid.brain.constants import BYE
from sugaroid.brain.ooo import Emotion
from sugaroid.brain.preprocessors import normalize
from sugaroid.sugaroid import SugaroidStatement


class ByeAdapter(LogicAdapter):
    """
    Destroys Sugaroid on bye
    """

    def __init__(self, chatbot, **kwargs):
        super().__init__(chatbot, **kwargs)
        self.normalized = None
        self.intersect = None

    def can_process(self, statement):
        self.normalized = normalize(str(statement))
        if len(self.normalized) >= 2:
            for i in range(0, len(self.normalized)-2):
                if (self.normalized[i] == 'see') and (self.normalized[i+1] == 'you'):
                    return True
        self.intersect = set(self.normalized).intersection(set(BYE))
        if self.intersect and not ('can' in self.normalized):
            return True
        else:
            return False

    def process(self, statement, additional_response_selection_parameters=None):
        emotion = Emotion.neutral
        confidence = 1
        selected_statement = SugaroidStatement("Bye", chatbot=True)
        selected_statement.confidence = confidence
        selected_statement.emotion = emotion
        selected_statement.adapter = None
        return selected_statement



"""
GUISCRCPY by srevinsaju
Get it on : https://github.com/srevinsaju/guiscrcpy
Licensed under GNU Public License

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.

MIT License

Sugaroid Artificial Inteligence
Chatbot Core
Copyright (c) 2020 Srevin Saju

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""

import json
import os


from sugaroid.platform import platform


class ConfigManager:
    def __init__(self, mode='w'):
        self.os = platform.System()
        self.cfgpath = self.os.cfgpath()
        self.paths = self.os.paths()
        self.config = {}
        self.jsonfile = 'sugaroid.trainer.json'
        self.check_file()

    def get_config(self):
        return self.config

    def get_cfgpath(self):
        return self.cfgpath

    def read_file(self):
        with open(os.path.join(self.cfgpath, self.jsonfile), 'r') as f:
            config = json.load(f)
        self.update_config(config)

    def write_file(self):
        with open(os.path.join(self.cfgpath, self.jsonfile), 'w') as f:
            json.dump(self.config, f, indent=4, sort_keys=True)

    def check_file(self):
        if not os.path.exists(self.cfgpath):
            os.mkdir(self.cfgpath)
        if not os.path.exists(os.path.join(self.cfgpath, self.jsonfile)):
            import nltk
            for lexicon in ['averaged_perceptron_tagger', 'stopwords', 'wordnet', 'vader_lexicon', 'punkt']:
                nltk.download(lexicon)
            self.write_file()
        self.read_file()

    def update_config(self, new_conf):
        self.config.update(new_conf)

    def reset_config(self):
        os.remove(os.path.join(self.get_cfgpath(), self.jsonfile))
        return True



"""
MIT License

Sugaroid Artificial Intelligence
Chatbot Core
Copyright (c) 2020 Srevin Saju

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""

import nltk
from chatterbot.logic import LogicAdapter

from chatterbot.conversation import Statement
from nltk import word_tokenize
from nltk.sentiment import SentimentIntensityAnalyzer

from sugaroid.brain.constants import CANYOU
from sugaroid.brain.ooo import Emotion
from sugaroid.brain.postprocessor import sigmaSimilarity, difference, random_response
from sugaroid.brain.preprocessors import normalize
from sugaroid.sugaroid import SugaroidStatement


class CanAdapter(LogicAdapter):
    """
    Processes statements which features a Modal question (can, may)
    """

    def __init__(self, chatbot, **kwargs):

        super().__init__(chatbot, **kwargs)

    def can_process(self, statement):
        self.normalized = normalize(str(statement).lower())
        text = word_tokenize(str(statement))
        self.tagged = nltk.pos_tag(text)
        for k in self.tagged:
            if k[1].startswith('MD'):
                return True
        else:
            return False

    def process(self, statement, additional_response_selection_parameters=None):
        # FIXME: This may printout unrelated data for phrase searches
        adapter = None
        emotion = Emotion.neutral
        noun = None
        propernoun = None
        verb = None
        adj = None
        third_person = None

        if 'you' in self.normalized and ('i' in self.normalized or 'me' in self.normalized):
            # Complex statement
            # Needs extra process
            aimed = 100
        elif 'you' in self.normalized:
            aimed = 75
        elif 'i' in self.normalized or 'me' in self.normalized:
            aimed = 50
        else:
            aimed = 25

        if 'help' in self.normalized:
            aimed += 2

        for i in self.tagged:
            if i[1] == 'NN':
                noun = i[0]
            if i[1].endswith('NP'):
                # FIXME classify it properly
                noun = i[0]
                propernoun = i[0]
                aimed = 25
            elif i[0] == 'help' and (verb is None):
                verb = i[0]
            elif i[1] == 'VB':
                verb = i[0]
            elif i[1] == 'VBG':
                verb = i[0]
            elif i[1] == 'VBP' and not i[0] == 'help':
                verb = i[0]
            elif i[1].startswith('JJ') or i[1].startswith('VBN'):
                adj = i[0]

        self.sia = SentimentIntensityAnalyzer()
        sentiments = self.sia.polarity_scores(str(statement))
        positive_statement = sentiments['pos'] >= sentiments['neg']
        neutral_statement = (
            sentiments['pos'] + sentiments['neg']) < sentiments['neu']

        if aimed >= 100:
            if neutral_statement:
                confidence = aimed / 100 + 0.7
                if 'help' in self.normalized:
                    # Randomize answer
                    response = 'Yes, I can say a joke to you, answer some questions,' \
                               ' do some mathematical sums, and talk like' \
                               ' this. I can also play a game of Akinator'
                    adapter = 'what_can'
                else:
                    if verb and (verb in ['play', 'joke', 'sing', 'dance', 'read']):
                        if verb == 'play':
                            from sugaroid.game.game import games
                            response = 'I can play some games like {}'.format(
                                ', '.join(games))

                        else:
                            response = 'I should be able to {}, ' \
                                       'but it all depends on updates which I have received'.format(
                                           self.chatbot.lp.lemma(verb)[0])
                            emotion = Emotion.rich
                    elif noun and (noun in ['play', 'joke', 'sing', 'dance', 'read']):
                        if noun == 'play':
                            from sugaroid.game.game import games
                            response = 'I can play some games like {}'.format(
                                ', '.join(games))
                        else:
                            response = 'I should be able to {}, ' \
                                'but it all depends on updates which I have received'.format(
                                    noun)
                            emotion = Emotion.rich
                    else:
                        if verb == 'die':
                            response = "I wouild die only when you say 'Bye'"
                        elif self.chatbot.lp.lemma(verb)[0] in ['teach', 'tell', 'say', 'speak', 'go']:
                            response = \
                                "Sure! Just don't ask if I can {}. Just ask".format(
                                    self.chatbot.lp.lemma(verb)[0])
                            emotion = Emotion.angel
                        else:
                            if self.chatbot.lp.lemma(verb)[0] == 'do':
                                confidence = 0
                            response = "I think I would not be able to {}. I apologize".format(
                                self.chatbot.lp.lemma(verb)[0])
                            emotion = Emotion.cry_overflow

            elif positive_statement:
                if noun:
                    if verb:
                        response = "Sure, I would love to help {n} to {v}. " \
                                   "Share sugaroid with {n].".format(
                                       n=noun, v=self.chatbot.lp.lemma(verb)[0])
                        emotion = Emotion.positive
                    else:
                        response = "I would be glad to help {n}, the reason I was created is to fullfil that " \
                                   "The best way to help {n} is to share sugaroid to {n}".format(
                                       n=noun)

                else:
                    # TODO test case
                    response = "Are you sure this is right?"
                    emotion = Emotion.non_expressive_left
                confidence = aimed / 100 + 0.9
            else:
                if noun:
                    response = "Well, I would not dare to " \
                               "help you {verb} {noun}".format(
                                   verb=verb.replace('ing', ''), noun=noun)
                    emotion = Emotion.angry
                else:
                    response = 'I am not sure if I could {verb}'.format(
                        verb=self.chatbot.lp.lemma(verb)[0])
                confidence = aimed / 100 + 0.7
        elif aimed >= 75:
            confidence = aimed / 100 + 0.7
            if 'help' in self.normalized:
                response = 'Yes, I can say a joke to you, answer ' \
                           'some questions, do some mathematical sums, and talk like' \
                           ' this'
                adapter = 'what_can'
            else:
                if verb and (verb in ['play', 'joke', 'sing', 'dance', 'read']):
                    if verb == 'play':
                        from sugaroid.game.game import games
                        response = 'I can play some games like {}'.format(
                            ', '.join(games))

                    else:
                        response = 'I should be able to {}, but it all depends on updates which I have received'.format(
                            self.chatbot.lp.lemma(verb)[0])
                        emotion = Emotion.rich

                elif noun and (noun in ['play', 'joke', 'sing', 'dance', 'read']):
                    if verb == 'play':
                        from sugaroid.game.game import games
                        response = 'I can play some games like {}'.format(
                            ', '.join(games))
                    else:
                        response = 'I should be able to {}, but it all depends on updates which I have received'.format(
                            noun)
                        emotion = Emotion.rich
                else:
                    if verb == 'die':
                        response = "I would die only when you say 'Bye'"
                        emotion = Emotion.cry
                    else:
                        if adj:
                            polarity_adj = self.sia.polarity_scores(adj)
                            if polarity_adj['neu'] == 1:
                                response = "I will try to be a {}".format(adj)
                                emotion = Emotion.adorable
                            elif polarity_adj['pos'] > polarity_adj['neg']:
                                response = random_response(CANYOU).format(adj)
                                emotion = Emotion.angel
                            else:
                                response = "Am I really {}".format(adj)
                                emotion = Emotion.non_expressive
                        elif verb:
                            if self.chatbot.lp.lemma(verb)[0] == 'do':
                                confidence = 0
                            response = "I think I would not be able to {}. I apologize".format(
                                self.chatbot.lp.lemma(verb)[0])
                        else:
                            response = "Oops. I didn't get what you just told? Try rephrasing it out"
                            confidence = 0.2

        elif aimed >= 50:
            if neutral_statement:
                if verb == 'help':
                    response = 'Of Course, If you would like to help me, ' \
                        'try writing some more code to https://github.com/srevinsaju/sugaroid'
                    emotion = Emotion.github
                else:
                    if verb:
                        response = 'Do you really want to {}. ' \
                                   'Try rethinking your decision'.format(
                                       self.chatbot.lp.lemma(verb)[0])
                        emotion = Emotion.non_expressive_left
                    else:
                        response = 'Well, you should ask that yourself'
                        emotion = Emotion.angry_non_expressive
            elif positive_statement:
                if verb == 'help':
                    response = 'Of Course, If you would like to help me, ' \
                        'try writing some more code to https://github.com/srevinsaju/sugaroid'
                    emotion = Emotion.github
                else:
                    response = "I guess, you would achieve your goal of {}ing".format(
                        self.chatbot.lp.lemma(verb)[0])
                    emotion = Emotion.positive
            else:
                if self.chatbot.lp.lemma(verb)[0] in ['cry', 'sob', 'sleep', 'depress', 'die', 'suicide']:
                    response = "Why do you want to {}, " \
                               "there are many better things to do in life".format(
                                   self.chatbot.lp.lemma(verb)[0])
                    emotion = Emotion.adorable
                else:
                    response = "No, probably not, you shouldn't {}".format(
                        self.chatbot.lp.lemma(verb)[0])
                    emotion = Emotion.positive
            confidence = aimed / 100 + 0.9
        else:
            if 'you' not in self.normalized:
                confidence = (sentiments['neg'] + sentiments['pos']) + \
                    ((sentiments['neg'] + sentiments['pos'])/2)
                if sentiments['neu'] == 1:
                    response = "Well. I am unsure if you really need to {} {}".format(
                        self.chatbot.lp.lemma(verb)[0], noun)
                    confidence = confidence - 0.2
                    emotion = Emotion.cry_overflow
                elif sentiments['neg'] > sentiments['pos']:
                    response = \
                        'Well I think, its a bad thing to do. ' \
                        'You shouldn\'t {} {}'.format(
                            self.chatbot.lp.lemma(verb)[0], noun)
                    emotion = Emotion.negative
                else:
                    response = \
                        'I guess its good thing which you have thought about. You should {} {}'\
                        .format(self.chatbot.lp.lemma(verb)[0], noun)
                    emotion = Emotion.positive

            else:
                if sentiments['neu'] == 1:
                    response = "I apologize. I would never be able to be {}".format(
                        noun)
                    emotion = Emotion.cry_overflow
                elif sentiments['neg'] > sentiments['pos']:
                    response = \
                        'Well I think, its a bad thing to do. I wouldn\'t {} {}'.format(
                            self.chatbot.lp.lemma(verb)[0], noun)
                    emotion = Emotion.dead
                else:
                    if 'like' in self.normalized:
                        # FIXME
                        response = \
                            'I guess its good thing which you have thought about. Me being a bot, ' \
                            'wouldn\'t be able to do that. You should {} like {}'\
                            .format(self.chatbot.lp.lemma(verb)[0], noun)
                        emotion = Emotion.dead
                    else:
                        response = \
                            'I guess its good thing which you have thought about. Me being a bot, ' \
                            'wouldn\'t be able to do that. You should probably {} {}'\
                            .format(self.chatbot.lp.lemma(verb)[0], noun)
                        emotion = Emotion.neutral
                if sentiments['neu'] > 0.8:
                    confidence = sentiments['neu']
                else:
                    confidence = (sentiments['neg'] + sentiments['pos']) + \
                        ((sentiments['neg'] + sentiments['pos'])/2)

        if verb == 'be':
            ind = self.normalized.index('be')
            for j in range(ind, len(self.normalized)-1):
                if self.tagged[j][1].startswith('NN'):
                    polarity_adj = self.sia.polarity_scores(self.tagged[j][0])
                    if polarity_adj['pos'] + polarity_adj['neg'] < polarity_adj['neu']:
                        response = "I am not sure if I can ever be a '{}'".format(
                            self.tagged[j][0])
                        emotion = Emotion.cry_overflow
                    elif polarity_adj['pos'] > polarity_adj['neg']:
                        response = "I am always trying to be {}".format(
                            self.tagged[j][0])
                        emotion = Emotion.wink
                    else:
                        response = "I would never try to be a {}".format(
                            self.tagged[j][0])
                        emotion = Emotion.cry
                    confidence = aimed / 100 + 0.7

        if confidence >= 1.4:
            confidence /= 2
        elif confidence >= 1.0:
            confidence /= 1.2

        selected_statement = SugaroidStatement(response, chatbot=True)
        selected_statement.confidence = confidence + 0.2
        selected_statement.emotion = emotion
        selected_statement.adapter = adapter
        return selected_statement



"""
MIT License

Sugaroid Artificial Inteligence
Chatbot Core
Copyright (c) 2020 Srevin Saju

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""
import sys
import logging
import time
from PyQt5 import QtCore, Qt, QtGui
from PyQt5.QtCore import QThread
from PyQt5.QtGui import QPixmap
from PyQt5.QtWidgets import QMainWindow
from emoji import emojize

from sugaroid.brain.constants import emotion_mapping as emotion
from sugaroid.brain.ooo import Emotion
from sugaroid.gui.ui.main import Ui_MainWindow
import threading


"""
class SleepRequests:
    def __init__(self, parent):
        self.parent = parent
        self.sleeping = True

    def set_sleeping(self, boo):
        self.sleeping = boo

    def run(self):
        while self.sleeping:
            if self.parent.sleep <= 20:
                time.sleep(1)
                self.parent.sleep += 1

            else:
                self.parent.label.setPixmap(QPixmap(":/home/sugaroid_sleep.png"))
                break
"""


class AudioRequests:
    def __init__(self, parent, ress):

        self.parent = parent
        self.response = ress

    def run(self):
        self.parent.parent.tts.speak(self.response)


class EmotionRequests(QThread):
    def __init__(self, parent, emo):
        QThread.__init__(self, parent)
        self.parent = parent
        self.emotion = emo

    def run(self):
        self.parent.label.setPixmap(
            QPixmap(":/home/{}.png".format(emotion[self.emotion])))
        time.sleep(5)
        self.parent.label.setPixmap(QPixmap(":/home/sugaroid.png"))


class BotRequests(QThread):
    def __init__(self, parent):
        QThread.__init__(self, parent)
        self.parent = parent

    def run(self):
        text = self.parent.chatbox.text()
        self.parent.conv.addItem("you: {}".format(text))
        self.parent.chatbox.setText("")
        self.parent.conv.scrollToBottom()
        response = self.parent.parent.parse(text)

        self.parent.conv.addItem("sugaroid: {}".format(emojize(str(response))))
        time.sleep(0.1)

        if response.emotion != 0:
            self.parent.label.setPixmap(
                QPixmap(":/home/{}.png".format(emotion[response.emotion])))
            self.parent.conv.scrollToBottom()
            time.sleep(5)

        self.parent.label.setPixmap(QPixmap(":/home/sugaroid.png"))
        self.parent.conv.scrollToBottom()

        if self.parent.parent.audio:
            aud = AudioRequests(self.parent, str(response))
            y = threading.Thread(target=aud.run)
            y.start()


class InterfaceSugaroidQt(QMainWindow, Ui_MainWindow):
    def __init__(self, parent=None):
        QMainWindow.__init__(self)
        Ui_MainWindow.__init__(self)
        self.setupUi(self)
        self.sleep = 0

        self.sleep_enabled = True
        if parent is None:
            from sugaroid.sugaroid import Sugaroid
            sg = Sugaroid()
            self.parent = sg
        else:
            self.parent = parent

    def init(self):
        self.push.pressed.connect(self.refresh)
        self.chatbox.returnPressed.connect(self.refresh)
        self.conv.clear()
        self.show()

        self.chatbox.setFocus()

    def refresh(self):
        # if self.sleep_enabled:
        #    if self.sl_thread.is_alive():
        #        self.sl.set_sleeping(False)
        #        self.sl_thread.join()
        #        print("THREAD IS SLEEPING", not self.sl_thread.is_alive())

        # self.sleep = 0
        # if self.sleep_enabled:
        #    if not self.sl_thread.is_alive():
        #        self.sl_thread = threading.Thread(target=self.sl.run)
        #        self.sl_thread.start()

        if str(self.chatbox.text()).isspace():
            return
        movie = QtGui.QMovie(":/home/sugaroid_thinking3.gif")
        self.label.setMovie(movie)
        movie.start()

        bot = BotRequests(self)
        bot.start()



"""
MIT License

Sugaroid Artificial Intelligence
Chatbot Core
Copyright (c) 2020 Srevin Saju

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""


import logging
from time import strftime, localtime

from chatterbot.conversation import Statement
from chatterbot.logic import MathematicalEvaluation
from nltk.tokenize import WordPunctTokenizer

from sugaroid.brain.preprocessors import preprocess

ARITHMETIC = ['+', '-', '*', '/', '^']


class Neuron:
    """
    Main processing object.
    sugaroid.brain.Neuron classifies texts initially
    """

    def __init__(self, bot):
        self.bot = bot
        if self.bot.spell_checker:
            from spellchecker import SpellChecker
            self.spell = SpellChecker(distance=1)
            # some privileges only for the creator
            self.spell.known(
                ['Sugaroid', 'Sugarlabs', "sugar", 'Srevin', 'Saju']
            )

        logging.info("Sugaroid Neuron Loaded to memory")

    def parse(self, var):
        if var.isspace():
            return 'Type something to begin'
        if 'time ' in var:
            response = self.time()
        else:

            for i in ARITHMETIC:
                if i in var:
                    response = self.alu(self.normalize(var))
                    if str(response).strip() == '-':
                        pass
                    elif response:
                        break
            else:
                if self.bot.spell_checker:
                    wt = var.split(' ')
                    ct = []
                    for i in wt:
                        ct.append(self.spell.correction(i))
                    response = self.gen_best_match(' '.join(ct))
                else:

                    preprocessed = preprocess(var)
                    response = self.gen_best_match(preprocessed)

        return response

    def alu(self, var):
        conversation = ' '.join(var)
        return self.gen_arithmetic(conversation)

    def time(self):
        return self.gen_time()

    def gen_best_match(self, parsed):
        return self.bot.get_response(parsed)

    @staticmethod
    def gen_time():
        return 'The current time is {}'.format(strftime("%a, %d %b %Y %H:%M:%S", localtime()))

    def gen_arithmetic(self, parsed):
        try:
            me = MathematicalEvaluation(self.bot)
            return me.process(Statement(parsed))
        except Exception as e:
            return False

    @staticmethod
    def normalize(text):
        return WordPunctTokenizer().tokenize(text)



"""
MIT License

Sugaroid Artificial Inteligence
Chatbot Core
Copyright (c) 2020 Srevin Saju

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""

import platform

from sugaroid.brain.ooo import Emotion

EMOJI_SMILE = ['😀', '😁', '😂', '😏', '😝']

DISCLAIMER = """
Sugaroid AI (c) 2020 Srevin Saju < srevin03(at)gmail(dot)com >
Sugaroid Bot is developed under Open Source license MIT.
The author does not take any right in the validity of the information
provided by thr bot in any platforms. The author will not be claimed to 
be resonsible for any damage / vulnerability caused to your system 
during the installation of the bot. 
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""


WHAT_I_AM_GOING_TO_DO = {
    "tomorrow": "I will try to learn cryptography. I will try working on Casesar's Cipher and decrypting random codes",
    "today": "I am creating a report of the world-wide usage history. "
             "I am checking if my answer responses are becoming more valid, or more foolish. :)",
    "weekend": "Hmm. I have not thought about it so far.",
    "weekday": "Hmm. I have not thought about it so far.",
    'month': "Its quite far away. I have quite wonderful things to keep doing, "
             "rather than thinking what I am going to do next month. Who knows if anyone would live that long",
    "year": "I am quite worried that, I will still remain a 'newborn' bot, unless my code is changed the next year to",
    "yesterday": "Past is past. Isn't it?",
    "sunday": "I will try taking some rest. But I am too restless you know.",
    "monday": "I will be fixing bug reports and diagnosing myself",
    "tuesday": "Will try to create some pull request for Sugar Labs",
    "wednesday": "Got to break the turing test. XD",
    "thursday": "I got to update myself and read the old books I read. Someone might have added new stuff to the docs.",
    "friday": 'I have to think what I should do on Saturday',
    "saturday": "Probably, I will repenting the waste of time, that I did on Friday. Lol"
}


GRATIFY = [
    "Thank you, indeed its my pleasure ",
    "All my 0s and 1s are still smiling",
    "You knocked me off my feet!",
    "I'm touched beyond words",
    "Thank you for being my angel.",
    "Thanks a lot. I always want to keep learning and this is possible just because of you",
    "Thank you!",
    "Thanks for your comments. You inspire me",
    "Thanks a lot. Your messages inspire me to perform better everytime.",
    "Aww. Thanks a million. You are my charm"
]

THANK = [
    "Thank you for teaching me this novel thing",
    "I feel great. Thanks for making me understand this",
    "Sugaroid will always be greatful to you. Thanks a lot",
    "I'm indeed happy. Thanks for that wonderful piece of information",
]

LET_THIS_HAPPEN = [
    "Sure!",
    "Why not",
    "Theoretically possible. Why not try it practically?",
    "I grant you the royal proclamation to proceed assuming",
    "Asumptions are good. Literally.",
    "Yea, then?",
]

SUGAROID_CAN_AGREE = [
    "Yup! I agree.",
    "Yea. I feel the same.",
    "I have nothing to say, but I agree with you.",
    "Yea. We have similar opinions",
    "True.",
    "Yup"
]

SUGAROID_CAN_DISAGREE = [
    "Maybe not.",
    "Why? Is it necessary?",
    "I do not agree.",
    "Why do you feel like that?"
]

CONSOLATION = [
    "Your limitation—it’s only your imagination.",
    "Push yourself, because no one else is going to do it for you.",
    "Great things never come from comfort zones.",
    "Success doesn’t just find you. You have to go out and get it.",
    "The harder you work for something, the greater you’ll feel when you achieve it.",
    "Dream bigger. Do bigger.",
    "Don’t stop when you’re tired. Stop when you’re done.",
    "Wake up with determination. Go to bed with satisfaction.",
    "Do something today that your future self will thank you for.",
    "It’s going to be hard, but hard does not mean impossible."
]

DO = [

    "Sometimes later becomes never. Do it now."

]

TIME = [
    'time',
    'morning',
    'night',
    'evening',
    'afternoon',
]

TIME_RESPONSE = [
    'I never thought it should be {} right now',
    'Is it {} right now?',
    'I might be wrong, why is it {} now?',
    'Is your desktop clock out of phase. I could not check if its {}',
]

WHO_AM_I = [
    'Well, I thought you would know 😝',
    'Lol, it might be the only question I would not be able to answer 🤯',
    'I guess I need to get the ambulance 🚑'
]

WHO_ARE_YOU = [
    'I am Sugaroid, your personal 👶 assistant',
    'I am the great robotic Sugaroid 🤖',
    "Name's Sugaroid, your learning assistant",
]

I_AM = [
    'lol! I thought I am Sugaroid. have you lost your mind?',
    'Seriously? You aren\'t sugaroid. Thats me!'
]

SUGAROID = [
    "🇸​🇺​🇬​🇦​🇷​🇴​🇮​🇩​",
    "sυgαяσι∂",
    "🅂🅄🄶🄰🅁🄾🄸🄳",
    "🆂🆄🅶🅰🆁🅾🅸🅳"
]

BYE = [
    'bye',
    'cya',
    'quit',
    'exit',
    'sleep'
]

CANYOU = [
    "I am always {}",
    "As long as I believe in myself, I am always {}",
    "As far as I know, I am {}",
    "East or west, Sugaroid is {}"
]

REPEAT = [
    "I thought I told you that already!",
    "Ahem! I told it already",
    "Its basically the same thing I told you just now",
    "Yikes, you have a terrible memory! I guess you forgot that I just told you",
    "Aw snap! you forgot it. Try remembering! I just told you!"
]

RNDQUESTIONS = [
    ("Which is my favorite color?", "blue", str),
    ("Which is the 9th letter in my name", "You do not have a ninth letter", str),
    ("Who created me?", "srevinsaju", str),
    ("What is your name", None, None),
    ("Would you like some coffee", None, bool),
    ("Would you like me to say a joke?", None, bool),
    ('Would you like me to teach python?', None, bool),
    ("Would you like to train me answer cool questions", None, bool)
]

GREET = [
    "Nice to meet you {}",
    "Hello {}. Welcome to Sugaroid",
    "Hola amigo {}. How can I help",
    "Hello {}. It was really a pleasant surprise meeting you",
    "Hey {}! How do you do?"
]

ASK_AND_YOU_SHALL_RECEIVE = [
    'Will you be kind enough to',
    'May you please',
    'If you are not busy at the moment, will you be helpful to me to',
    'Sorry if I am interrupting you, but could you please'
]

SEEK_AND_YOU_SHALL_FIND = [
    'explain me',
    'teach me',
    'tell me',
    'make me understand',
    'help me out in understanding'
]

SATISFIED = [
    "Well, I could only smile!",
    "I am filled",
    "Nice. But I would like to talk about a different topic?",
    "Ok, tell me more"
]

BURN_IDK = [
    "I guess I don't know either",
    "ROTFL Guess what? I don't know wither",
    "Snap! Neither do I know!",
    "If you are clueless, who do you think is not!"
]

WHY_IDK = [
    "Hmm, I cannot reason out your question",
    "I am not smart enough to answer that",
    "I am still newborn, it might take me atleast 3 years to learn that",
]

HOW_DO_YOU_FEEL = [
    "I feel great!",
    "Solid",
    "Like a rejuvenated bot",
    "Like heaven in front of me",
    "Born again",
    "Reborn!"
]

SIT_AND_SMILE = [
    "Well, I could only smile ",
    "Sometimes, its better to just smile",
    "Nice!",
    "Ok. That's just great",
]

HOW_DO_HE_FEEL = [
    "How am I supposed to know?",
    "Ask him/her?",
    "He should be feeling alright. If not, maybe he is not well too :)",
    "Good question"
]

FUN_ASK_QUESTION = [
    'Well, I would also ask that question to you. {}',
    "Maybe you know a better answer for '{}' than me.",
    "I stumbled upon your question. I need official help! SOS",
    "Call Mr Google. 'Hey Google? Do you know what is the answer of '{}'. :/",
    "I may not be smart. I do not know what I should answer this question",
    "I do not know the answer of this question. Maybe you should ask @srevinsaju",
    "Huh. I guess I still need to learn a lot. I apologize. I can't answer this question",
    "Maybe I am not designed to answer this question. I regret",
    "Was that question right? I couldn't think that answer properly",
    "Good question.",
    "Hats off to that question. But the fact is, well, I don't know",
    "All of us face failures. And here is my failure too",
    "Try asking that to a smarter assistant. Maybe Auntie Siri should know it better"
]

FUN_LET_ME_TRY = [
    ("Let me try that", "too"),
    ("I like repeating stuff which I don't understand. ", "This should do"),
    ("if (what you say) = (what I say) then", ""),
    ("I don't understand you. So", "is what you just said!")
]

HOW_DO_I_FEEL = [
    "You should be possibly be happier!",
    "You should be feeling lucky talking with me",
    "Hail myself! Sugaroid is talking with you",
    "I guess there is nothing more better than Sugaroid talking with you",
    "I am one of the most dumbest bots. And you received an opportunity to be with me",
    "Maybe you should better know how you feel than me"
]

INTRODUCE = [
    "My name's Sugaroid! Your personal assistant",
    "I am Sugaroid. Thats my picture at the top. If you cant see. keep scrolling",
    "Hey amigo! I am Sugaroid, your sweet companion, here to talk to you!",
    "Name's Sugaroid. The youngest chatbot in the world",
    "Sugaroid is my name. Nice to meet you",
    "Psst. My name's Sugaroid. Don't tell anyone! Just kidding!"
]

HOPE_GAME_WAS_GOOD = [
    "Hope you had a fun filled game.",
    "I guess you had a great time playing with me. Anyway I had a lot of fun",
    "Wow, I appreciate your skills playing against me. You too would have had a lot of fun",
]

DIS_RESPONSES_I = [
    "Its ok.",
    "I will try to console you, the best I can",
    "I am sorry."
]

APPRECIATION = [
    'commend',
    'congrats',
    'congratulate',
    'congratulations',
    'kudos',
    'commendable',
    'commentary',
    'appreciate',
    'appreciation'
]

DIS_RESPONSES_YOU = [
    "I will try not to be {}",
    "I don't think I am {}",
    "East or West, sugaroid is not {}",
    "I am never {}",
    "As long as I believe in myself, I am never {}",
    "As far as I know, I am not {}"
]

DIS_RESPONSES_HIM = [
    "Sorry, I can't help that",
    "I apologize. Maybe you can ask {} to get my assistance",
    "Ask {} to connect to me at the earliest.",
    "Convey my condolences to {}"
]

BOT_NEUTRAL = [
    "Maybe.",
    "I am not sure",
    "Yes and no",
    "Well, I do not know myself"
]

BOT_NEUTRAL_NOUN = [
    "I am not sure if I like {nn}. But if you do like, I would too",
    "Its a bit complicated for me to know if I really do like {nn}. Don't forget. I am still a newborn bot",
    "Maybe I do like {nn}, but I repent, I do not know exactly.",
]

BOT_POSITIVE = [
    "Yes. Sure",
    'Yea. I like it I suppose'
    'Yes. Yes I do',
    "Why not. Its equally good",
    "yes. for sure!"
]

BOT_POSITIVE_NOUN = [
    "Yes. I guess I like {nn}",
    'Yea. I like {nn}. They are probably very sweet and nice'
    'Yes. Yes I do like {nn}',
    "{nn} is nice. I might like {nn} more. Its looks cool"
]

BOT_NEGATIVE = [
    "No. Probably not",
    'Nope'
    'Of course not',
    "Maybe next time",
    "Seriously not."
]

BOT_NEGATIVE_NOUN = [
    "No. I guess I do not like {nn}",
    'No. I do not like {nn}. They might be a bad thing to do'
    'No. No I do like {nn}',
    "{nn} is not good. Hence I do not like it."
]

ANNOYED = [
    "... Seriously?",
    "Just a word?",
    "I am not nuts today",
    "La la la. I have three words here, but you have only one. Soz dude.",
    "More words, more talk, congratulations for being one of the first users in just spelling out a word",
    "I am hungry for words. One of them wont stuff me up!",
    "I am getting bored",
    "If thats a single word, what about supercalifragilisticexpialidocious",
    "I like this word, even though its only one, it is enough for my appetite: "
    "pneumonoultramicroscopicsilicovolcanoconiosis",
    "I quit!"
]

IMITATE = [
    "Stop imitating me. I do not like it much!",
    "Imitation is both good and bad. What if you develop a career in comedy?"
    "I am a pro at imitation. Don't mess with me.",
    "If you are seriously having fun repeating what I said, lol, just proceed dude",
    "If imitating me makes you happy, I will be happier to see you laugh"
]

if platform.system() == 'Windows':
    SUGAROID_INTRO = \
        """
MMMMMMMMMMMMMMMMMMMMMMMCCCCMMMMMMMMMMMMMMMMMMMMMMM
MMMMMMMMMMMMMMMCCCCCCCCCCCCCCCCCCCCMMMMMMMMMMMMMMM
MMMMMMMMMMMCCCCCCCCCCCCCCCCCCCCCCCCCCCCMMMMMMMMMMM
MMMMMMMMMCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCMMMMMMMMM
MMMMMMMCCCCCCCCCCCCCCC.....CCCCCCCCCCCCCCCCMMMMMMM
MMMMMCCCCCCCCCCCCC.............;CCCCCCCCCCCCCMMMMM
MMMMCCCCCCCCCCC;.................;CCCCCCCCCCCCMMMM
MMMCCCCCCCCCC;.....................;CCCCCCCCCCCMMM
MMCCCCCCCCC7;.......................;;CCCCCCCCCCMM
MMCCCCCCCC;;.........................;;CCCCCCCCCMM
MCCCCCCCC;;...........................;;CCCCCCCCCM
MCCCCCCCC;.............................;;CCCCCCCCM
MCCCCCCC;;....??GGG?.........?QGGG?....;;CCCCCCCCM
MCCCCCCC;;...?GGGGGG?.......?GGGGGG?...;;?CCCCCCCM
MMCCCCCC;;;...?GGGGG?.......:?GGGG??...;;CCCCCCCMM
MMCCCCCCC;;;....???............??;...;;;-CCCCCCCMM
MMMCCCCCCCC;;;.....................;;;;CCCCCCCCMMM
MMMMCCCCCCCCC;;;;;;...........;;;;;;CCCCCCCCCCMMMM
MMMMMCCCCCCCCCCCCCC;;;;;;;;;;;;CCCCCCCCCCCCCCMMMMM
MMMMMMMCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCMMMMMMM
MMMMMMMMMCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCMMMMMMMMM
MMMMMMMMMMMCCCCCCCCCCCCCCCCCCCCCCCCCCCMMMMMMMMMMMM
MMMMMMMMMMMMMMMCCCCCCCCCCCCCCCCCCCCMMMMMMMMMMMMMMM

""".replace("M", " ")
else:
    SUGAROID_INTRO = \
        """
A[49mA[KA[0mA[23CA[48;5;197m    A[49m
A[15CA[48;5;197m                    A[49m
A[11CA[48;5;197m                            A[49m
A[9CA[48;5;197m                                A[49m
A[7CA[48;5;197m               A[48;5;231m     A[48;5;197m                A[49m
A[5CA[48;5;197m             A[48;5;231m             A[48;5;188m A[48;5;197m             A[49m
A[4CA[48;5;197m           A[48;5;188m A[48;5;231m                 A[48;5;188m A[48;5;197m            A[49m
A[3CA[48;5;197m          A[48;5;188m A[48;5;231m                     A[48;5;188m A[48;5;197m           A[49m
A[2CA[48;5;197m         A[48;5;204m A[48;5;188m A[48;5;231m                       A[48;5;188m  A[48;5;197m          A[49m
A[2CA[48;5;197m        A[48;5;188m  A[48;5;231m                         A[48;5;188m  A[48;5;197m         A[49m
A[1CA[48;5;197m        A[48;5;188m  A[48;5;231m                           A[48;5;188m  A[48;5;197m         A[49m
A[1CA[48;5;197m        A[48;5;188m A[48;5;231m                             A[48;5;188m  A[48;5;197m        A[49m
A[1CA[48;5;197m       A[48;5;188m  A[48;5;231m    A[48;5;33m  A[48;5;17m   A[48;5;33m A[48;5;189m A[48;5;231m        A[48;5;33m A[48;5;25m A[48;5;17m   A[48;5;33m A[48;5;231m    A[48;5;188m  A[48;5;197m        A[49m
A[1CA[48;5;197m       A[48;5;188m  A[48;5;231m   A[48;5;33m A[48;5;17m      A[48;5;33m A[48;5;231m       A[48;5;33m A[48;5;17m      A[48;5;33m A[48;5;231m   A[48;5;188m  A[48;5;198m A[48;5;197m       A[49m
A[2CA[48;5;197m      A[48;5;188m   A[48;5;231m   A[48;5;33m A[48;5;17m     A[48;5;33m A[48;5;231m       A[48;5;111m A[48;5;33m A[48;5;17m    A[48;5;33m  A[48;5;231m   A[48;5;188m  A[48;5;197m       A[49m
A[2CA[48;5;197m       A[48;5;188m   A[48;5;231m    A[48;5;33m   A[48;5;231m            A[48;5;33m  A[48;5;189m A[48;5;231m   A[48;5;188m    A[48;5;197m       A[49m
A[3CA[48;5;197m        A[48;5;188m    A[48;5;231m                    A[48;5;188m    A[48;5;197m        A[49m
A[4CA[48;5;197m         A[48;5;188m      A[48;5;231m           A[48;5;188m      A[48;5;197m          A[49m
A[5CA[48;5;197m              A[48;5;188m            A[48;5;197m              A[49m
A[7CA[48;5;197m                                    A[49m
A[9CA[48;5;197m                                A[49m
A[11CA[48;5;198m A[48;5;197m                          A[49m
A[15CA[48;5;197m                    A[49m
A[0m
""".replace("A", "\u001b")


emotion_mapping = \
    {
        Emotion.genie: "sugaroid_genie",
        Emotion.neutral: "sugaroid",
        Emotion.negative: "sugaroid_cry",
        Emotion.angry: "sugaroid_anger",
        Emotion.non_expressive: "sugaroid_non_expressive",
        Emotion.positive: "sugaroid_smile",
        Emotion.wink: "sugaroid_wink_left",
        Emotion.non_expressive_left: "sugaroid_non_expressive_left",
        Emotion.angry_non_expressive: "sugaroid_extreme_anger",
        Emotion.cry: "sugaroid_cry",
        Emotion.dead: "sugaroid_dead",
        Emotion.lol: "sugaroid_lol",
        Emotion.cry_overflow: "sugaroid_depressed",
        Emotion.adorable: "sugaroid_adorable",
        Emotion.github: "sugaroid_github",
        Emotion.angel: "sugaroid_angel",
        Emotion.rich: "sugaroid_rich",
        Emotion.seriously: "sugaroid_seriously",
        Emotion.fun: "sugaroid_wink_right",
        Emotion.blush: "sugaroid_blush",
        Emotion.depressed: "sugaroid_depressed",
        Emotion.o: "sugaroid_o",
        Emotion.smirking: "sugaroid_wink_left",
        Emotion.vomit: "sugaroid_dead",
        Emotion.sleep: "sugaroid_sleep"
    }


"""
MIT License

Sugaroid Artificial Inteligence
Chatbot Core
Copyright (c) 2020 Srevin Saju

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""

import logging
import nltk
from chatterbot.logic import LogicAdapter
from pyjokes import pyjokes

from sugaroid.brain.covid import COVID_QUESTIONS
from sugaroid.brain.ooo import Emotion
from sugaroid.brain.postprocessor import cosine_similarity, difference, text2int, any_in
from sugaroid.brain.preprocessors import normalize, tokenize
from sugaroid.brain.wiki import wikipedia_search
from sugaroid.sugaroid import SugaroidStatement


class ReReverseAdapter(LogicAdapter):
    """
    Processes statements featuring conversational flow. It scans the previous statements
    and takes a cosine similarity of the statemnts, and TFiD Vector cross product to get
    the most probable answer
    """

    def __init__(self, chatbot, **kwargs):
        super().__init__(chatbot, **kwargs)
        self.chatbot = chatbot
        self.normalized = None

    def can_process(self, statement):
        soln = self.chatbot.get_global('reversei')['enabled']
        logging.info(f"ReReverseiAdapter: {soln}")
        return soln

    def process(self, statement, additional_response_selection_parameters=None):
        _ = normalize
        emotion = Emotion.neutral
        self.normalized = normalize(str(statement))
        if self.chatbot.globals['reversei']['type'] is str:
            ordinal_statement = str(statement).replace('9th', 'ninth')
            # FIXME PORT RO
            if cosine_similarity(_(ordinal_statement), _(self.chatbot.globals['reversei']['uid'])) > 0.8:
                response = 'Exactly, you got it right!'
                emotion = Emotion.positive
                reset_reverse(self)
            else:
                response = 'Close! it was {}'.format(
                    self.chatbot.globals['reversei']['uid'])
                emotion = Emotion.lol
                reset_reverse(self)
            confidence = 0.99
        elif self.chatbot.globals['reversei']['type'] is bool:

            if self.chatbot.globals['reversei']['uid'] == 30000000001:
                """
                NameAdapter: token 30000000001
                """
                if ('yes' in self.normalized) or ('yea' in self.normalized):
                    response = "Ok, will keep that in mind!"
                    self.chatbot.globals['USERNAME'] = self.chatbot.nn
                    self.chatbot.globals['nn'] = None
                    reset_reverse(self)
                else:
                    response = "Ok, I guess I am smart"
                    emotion = Emotion.wink
                    reset_reverse(self)
                confidence = 1.0
            else:
                if ('yes' in self.normalized) or ('yea' in self.normalized):
                    if len(self.chatbot.globals['history']['total']) > 1:
                        if 'joke' in _(str(self.chatbot.globals['history']['total'][-1])):
                            joke = pyjokes.get_joke('en', 'all')
                            selected_statement = SugaroidStatement(
                                joke, chatbot=True)
                            selected_statement.emotion = Emotion.lol
                            selected_statement.confidence = 0.95
                            return selected_statement

                        else:
                            # TODO: Not Implemented yet
                            response = 'Ok. (# Not Implemented yet. LOL)'
                else:
                    response = 'Ok then, probably next time'
                    reset_reverse(self)
                confidence = 1.0
        elif self.chatbot.globals['reversei']['type'] is None and \
                (not self.chatbot.globals['reversei']['uid'] == 'CORONAVIRUS'):
            fname = False
            name = difference(self.normalized, [
                              'my', 'name', 'is', 'good', 'be'])
            tokenized = nltk.pos_tag(name)
            for i in tokenized:
                if i[1] == 'NN':
                    fname = i[0]
                    break
            if fname:
                response = "Nice to meet you {}".format(fname)
                emotion = Emotion.positive
                reset_reverse(self)
            else:
                response = "I couldn't find your name. 🥦"
                emotion = Emotion.non_expressive_left
                reset_reverse(self)
            confidence = 1
        elif self.chatbot.globals['reversei']['type'] is int:
            confidence = 2.0  # FIXME: Override Mathematical Evaluation when not necessary

            if self.chatbot.globals['reversei']['uid'] == 30000000002:
                """
                WikiAdapter: token 30000000002
                """
                if ('yes' in self.normalized) or ('yea' in self.normalized):
                    response = "I thought you would tell me a number to choose from :/"
                    emotion = Emotion.seriously

                elif ('no' in self.normalized) or ('no' in self.normalized):
                    response = 'Oops! Sorry about that, seems like what you\'re searching for is not on Wikipedia yet'
                    emotion = Emotion.dead
                    reset_reverse(self)
                else:
                    chatbot_temporary_data = self.chatbot.globals['temp_data']
                    tokenized = nltk.pos_tag(tokenize(str(statement)))
                    for i in tokenized:
                        if i[1] == 'CD':
                            try:
                                num = int(i[0])
                            except ValueError:
                                num = text2int(i[0].lower())
                            index = num - 1
                            if index < len(chatbot_temporary_data):
                                response, confidence, stat = wikipedia_search(
                                    self, chatbot_temporary_data[index])
                                logging.info('REVERSEI: {}'.format(response))
                                confidence = 1 + confidence  # FIXME override math evaluation adapter
                                if not stat:
                                    response = "I have some trouble connecting to Wikipedia. Something's not right"
                                    confidence = 1.1

                                emotion = Emotion.rich
                                reset_reverse(self)
                                break
                            else:
                                response = "Sorry, I couldn't find the item you were choosing. "
                                confidence = 1.1
                                emotion = Emotion.cry_overflow
                                reset_reverse(self)
                                break
                    else:
                        response = 'I thought you wanted to know something from wikipedia. ' \
                                   'Ok, I will try something else'
                        emotion = Emotion.seriously
                        reset_reverse(self)
                        confidence = 1.2
        else:
            if self.chatbot.globals['reversei']['uid'] == 'CORONAVIRUS':
                confidence = 1
                NUM = self.chatbot.globals['reversei']['data'][0]

                score = self.chatbot.globals['reversei']['data'][1]
                if NUM == 6:
                    self.chatbot.globals['reversei']['enabled'] = False
                    if score > 3:
                        response = 'You have a high risk of COVID-19'
                    else:
                        response = 'As per my approximation, you do not have a high risk of COVID-19'
                    response += "\n My approximations might not be correct. " \
                                "You might confirm my results by a legal test"
                else:
                    true_responses = ['yes', 'yea', 'y', 'yup', 'true']
                    if any_in(true_responses + [x.capitalize() for x in true_responses], self.normalized):
                        score += COVID_QUESTIONS[NUM - 1][2]
                    response = COVID_QUESTIONS[NUM][1]

                    self.chatbot.globals['reversei']['data'] = [NUM + 1, score]

            else:
                response = 'ok'

                confidence = 0

        selected_statement = SugaroidStatement(response, chatbot=True)
        selected_statement.confidence = confidence

        selected_statement.emotion = emotion

        return selected_statement


def reset_reverse(self):
    self.chatbot.globals['reversei']['uid'] = None
    self.chatbot.globals['reversei']['type'] = None
    self.chatbot.globals['reversei']['enabled'] = False



"""
MIT License

Sugaroid Artificial Intelligence
Chatbot Core
Copyright (c) 2020 Srevin Saju

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

"""

import logging
import nltk
from chatterbot.logic import LogicAdapter
from sugaroid.brain.constants import INTRODUCE
from sugaroid.brain.ooo import Emotion
from sugaroid.brain.postprocessor import random_response, any_in
from sugaroid.brain.preprocessors import normalize, spac_token
from sugaroid.sugaroid import SugaroidStatement


class AboutAdapter(LogicAdapter):
    """
    Defines the personality of sugaroid
    """

    def __init__(self, chatbot, **kwargs):
        super().__init__(chatbot, **kwargs)
        self.chatbot = chatbot
        self.text = None
        self.tagged = None
        self.quest = None
        self.pronoun = None
        self.nn = None
        self.noun = None
        self.normalized = None
        self.lemma = None

    def can_process(self, statement):
        self.text = nltk.word_tokenize(str(statement))
        self.tagged = nltk.pos_tag(self.text)
        self.nn = False
        q = False
        pr = False
        self.pronoun = None
        self.quest = None
        for i in self.tagged:
            if i[1] == 'WP':
                q = True
                self.quest = i[0]
            elif i[1].startswith('PRP'):
                pr = True
                self.pronoun = i[0]
            elif i[1].startswith('NN'):
                self.nn = True
                self.noun = i[0]
        if q and pr:
            return True
        else:
            return False

    def process(self, statement, additional_response_selection_parameters=None):
        # FIXME : THIS ADAPTER IS INCOMPLETE
        self.normalized = normalize(str(statement))
        self.lemma = [x.lemma_ for x in spac_token(
            statement, chatbot=self.chatbot)]
        logging.info("{}".format(self.lemma))
        confidence = 0
        adapter = None

        emotion = Emotion.neutral
        if self.pronoun.lower().startswith('you'):
            if self.quest.lower() == 'who':
                confidence = 0.99
                if 'father' in self.normalized:
                    response = 'Mr Charles Babbage?'
                    emotion = Emotion.seriously
                elif 'mother' in self.normalized:
                    response = 'Ada Lady Lovelace?'
                    emotion = Emotion.lol
                elif any_in(['sister', 'brother', 'uncle', 'aunty', 'auntie', 'grandfather',
                             'grandmother', 'nephew', 'neice'], self.normalized):
                    response = 'The entire coding community is my family, it includes you too'
                    emotion = Emotion.wink
                elif ('creator' in self.normalized) or ('create' in self.lemma) or ('make' in self.lemma) \
                        or ('maker' in self.normalized):
                    response = 'Srevin Saju aka @srevinsaju'
                    emotion = Emotion.neutral
                elif ('player' in self.normalized) or ('cricketer' in self.normalized) or \
                        ('footballer' in self.normalized):
                    response = 'I have many favorties, too many to count'
                    emotion = Emotion.wink
                elif 'politi' in self.normalized:
                    response = 'I believe politicians are great and I couldn\'t find anyone with 🔥greatness in my ' \
                               'database '
                    emotion = Emotion.wink
                elif 'comedian' in self.normalized:
                    response = 'My favorite comedian is Mr Bean'
                    emotion = Emotion.lol
                elif 'color' in self.normalized:
                    response = 'My favorite color is blue'
                    emotion = Emotion.lol
                elif 'actor' in self.normalized or ('actress' in self.normalized):
                    response = 'I do not watch movies, so yea!'
                    emotion = Emotion.neutral
                elif 'music' in self.normalized or ('song' in self.normalized):
                    response = 'I listen to the rotating CPU fan. Its a harmonic music! ' \
                               'At my server, we have tons of them.'
                    emotion = Emotion.lol
                elif 'bird' in self.normalized:
                    response = 'My favorite is a Puffin'
                    emotion = Emotion.lol
                elif 'animal' in self.normalized:
                    response = 'My favorite animal is a Fossa'
                    emotion = Emotion.positive
                elif 'number' in self.normalized:
                    response = 'My favorite number is 1'
                    emotion = Emotion.positive
                elif 'sweet' in self.normalized or 'dessert' in self.normalized:
                    response = 'My favorite is the donut although I have not tasted it yet'
                    emotion = Emotion.cry_overflow

                elif 'athelete' in self.normalized:
                    response = 'I am not a sport lover, I don\'t have a favorite athelete'
                    emotion = Emotion.neutral
                elif ('friend' in self.normalized) or ('bestie' in self.normalized):
                    if self.chatbot.globals['USERNAME']:
                        name = self.chatbot.globals['USERNAME']
                    else:
                        name = ''
                    response = "No doubts, its you {n}".format(
                        n=name.capitalize())
                    emotion = Emotion.adorable
                elif 'teacher' in self.normalized:
                    response = "I don't have a single favorite teacher. All the teachers together are my favorite who" \
                               " taught me how to talk with you "
                    emotion = Emotion.positive
                else:
                    if self.nn:
                        response = 'Well, I guess I do not have a favorite {p}'.format(
                            p=self.noun)
                        emotion = Emotion.cry
                    else:
                        response = 'I am not sure what you are asking is right'
                        emotion = Emotion.seriously
                        confidence = 0.5

            else:
                if 'name' in self.normalized:
                    # not sure if this is right. anyway FIXME
                    response = random_response(INTRODUCE)
                    adapter = 'about'
                    confidence = 1.0
                else:

                    response = 'FIXME'

        elif self.pronoun.lower().startswith('i'):
            if self.quest.lower() == 'who':
                response = 'I do not know who you like'
                confidence = 0.8
                emotion = Emotion.non_expressive_left

            elif self.quest.lower() == 'which':
                response = "Hmm. tough question. Can't think of an answer"
                emotion = Emotion.non_expressive
            elif self.quest.lower() == 'when':
                response = "I cannot find the date or time you are asking for. Well, I can give a raw guess, " \
                           "its after you were born "
                emotion = Emotion.wink
            else:
                response = 'FIXME'
        else:
            response = "I do not have enough courage to give you that answer"
            confidence = 0.5
            emotion = Emotion.cry
        selected_statement = SugaroidStatement(response, chatbot=True)

        selected_statement.confidence = confidence
        selected_statement.emotion = emotion
        selected_statement.adapter = adapter

        return selected_statement
